/*! 
 * @file fi_fpga_regs.h
 * @brief Definitions for the Fastec Imaging FPGA registers.
 *
 * Copyright (C) 2011,2015 Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef __FI_FPGA_REGS_H__
#define __FI_FPGA_REGS_H__

// Define the start and end of the FPGA register space.
#define FI_FPGA_BASE_ADDR  0		// Base physical address
#define FI_FPGA_MAX_ADDR   0x13000

// Currently all sections are offset from the same base address.
#define FI_FPGA_MAIN_OFFSET     0x00000
#define FI_FPGA_ASMI_OFFSET	0x01000
#define FI_FPGA_IM_ARB_OFFSET	0x04000
#define FI_FPGA_ICM_OFFSET	0x08000
#define FI_FPGA_IPM0_OFFSET	0x0C000
#define FI_FPGA_IPM1_OFFSET	0x10000

//-----------------------------------------------------------------------------
// Constant Definitions
//-----------------------------------------------------------------------------

//-------------------------------------
// Main Register Space
//-------------------------------------
#define FI_FPGA_MAIN_REG(x) (FI_FPGA_BASE_ADDR + FI_FPGA_MAIN_OFFSET + (x))

#define NC1_FPGA_CONTROL		FI_FPGA_MAIN_REG(0x0000)
#define NC1_FPGA_STATUS			FI_FPGA_MAIN_REG(0x0004)

#define TEST_REG2			FI_FPGA_MAIN_REG(0x0008)
#define TEST_REG3			FI_FPGA_MAIN_REG(0x000C)
#define TEST_REG4			FI_FPGA_MAIN_REG(0x0010)
#define TEST_REG5			FI_FPGA_MAIN_REG(0x0014)
#define TEST_REG6			FI_FPGA_MAIN_REG(0x0018)
#define TEST_REG7			FI_FPGA_MAIN_REG(0x001C)

#define FPGA_REVISION			FI_FPGA_MAIN_REG(0x0020)

#define PCIE_IRQ_TEST			FI_FPGA_MAIN_REG(0x0024)
#define PCIE_BASE_ADDR_HI		FI_FPGA_MAIN_REG(0x0030)
#define PCIE_BASE_ADDR_LO		FI_FPGA_MAIN_REG(0x0034)
#define PCIE_IRQ_MASK	 		FI_FPGA_MAIN_REG(0x0038)
#define PCIE_IRQ_STATUS 		FI_FPGA_MAIN_REG(0x003C)

#define PCIE_AUX_WR_DMA_CONTROL		FI_FPGA_MAIN_REG(0x0040)
#define PCIE_AUX_WR_DMA_STATUS		FI_FPGA_MAIN_REG(0x0044)
#define PCIE_AUX_WR_DMA_START_ADDR	FI_FPGA_MAIN_REG(0x0048)
#define PCIE_AUX_WR_DMA_BLOCK_SIZE	FI_FPGA_MAIN_REG(0x004C) // in QWORDS, 4x4=16 bytes each
#define PCIE_AUX_WR_DMA_BURST_SIZE	FI_FPGA_MAIN_REG(0x0050) // in QWORDS
#define PCIE_AUX_WR_DMA_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0054)
#define PCIE_AUX_WR_DMA_BURST_INTERVAL	FI_FPGA_MAIN_REG(0x0058)
#define PCIE_AUX_WR_DMA_PATTERN_VAL	FI_FPGA_MAIN_REG(0x005C)

#define PCIE_AUX_RD_DMA_CONTROL		FI_FPGA_MAIN_REG(0x0080)
#define PCIE_AUX_RD_DMA_STATUS		FI_FPGA_MAIN_REG(0x0084)
#define PCIE_AUX_RD_DMA_START_ADDR	FI_FPGA_MAIN_REG(0x0088)
#define PCIE_AUX_RD_DMA_BLOCK_SIZE	FI_FPGA_MAIN_REG(0x008C) // in QWORDS, 4x4 words = 16 bytes
#define PCIE_AUX_RD_DMA_BURST_SIZE	FI_FPGA_MAIN_REG(0x0090) // in QWORDS
#define PCIE_AUX_RD_DMA_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0094)
#define PCIE_AUX_RD_DMA_BURST_INTERVAL	FI_FPGA_MAIN_REG(0x0098)
#define PCIE_AUX_RD_DMA_PATTERN_VAL	FI_FPGA_MAIN_REG(0x009C)
#define PCIE_AUX_RD_DMA_ERROR_COUNT	FI_FPGA_MAIN_REG(0x00A0)

// Bob calls these TIMEKEEPER_... instead of TIME_...
#define TIME_CONTROL                    FI_FPGA_MAIN_REG(0x00C0)
#define TIME_STATUS                     FI_FPGA_MAIN_REG(0x00C4)
#define TIME_SBS_LOAD_VALUE             FI_FPGA_MAIN_REG(0x00C8)
#define TIME_SBS_OFFSET_VALUE           FI_FPGA_MAIN_REG(0x00CC)
#define TIME_1MHZ_TUNING_WORD_MASTER    FI_FPGA_MAIN_REG(0x00D0)
#define TIME_1MHZ_TUNING_WORD_SLAVE     FI_FPGA_MAIN_REG(0x00D4)
#define TIME_SBS_COUNTER_LIVE           FI_FPGA_MAIN_REG(0x00D8)
#define TIME_SBUS_COUNTER_LIVE          FI_FPGA_MAIN_REG(0x00DC)
#define TIME_SBS_COUNTER_LATCHED        FI_FPGA_MAIN_REG(0x00E0)
#define TIME_SBUS_COUNTER_LATCHED       FI_FPGA_MAIN_REG(0x00E4)

#define EXT_IO_CONTROL			FI_FPGA_MAIN_REG(0x0100)
#define EXT_IO_STATUS			FI_FPGA_MAIN_REG(0x0104)
#define TRIG_OUT_PULSE_WIDTH		FI_FPGA_MAIN_REG(0x0108)
#define SYNC_OUT_PULSE_WIDTH		FI_FPGA_MAIN_REG(0x010C)
#define ARM_OUT_PULSE_WIDTH		FI_FPGA_MAIN_REG(0x0110)
#define TRIG_IN_DEBOUNCE_TIME		FI_FPGA_MAIN_REG(0x0114)
#define SYNC_IN_DEBOUNCE_TIME		FI_FPGA_MAIN_REG(0x0118)
#define ARM_IN_DEBOUNCE_TIME		FI_FPGA_MAIN_REG(0x011C)
#define SYNC_IN_PERIOD			FI_FPGA_MAIN_REG(0x0120)
#define SYNC_IN_PERIOD_MIN		FI_FPGA_MAIN_REG(0x0124)
#define TRIGBTN_IN_DEBOUNCE_TIME	FI_FPGA_MAIN_REG(0x0128)
#define EXT_IO_MUX_CONTROL		FI_FPGA_MAIN_REG(0x012C)

#define PCIE_LATENCY_TIMER		FI_FPGA_MAIN_REG(0x0140)
#define PCIE_LATENCY_HICCUP_THRESHOLD	FI_FPGA_MAIN_REG(0x0144)
#define PCIE_WR_MIN_TIME_BETWEEN_HICCUPS	FI_FPGA_MAIN_REG(0x0148)
#define PCIE_RD_MIN_TIME_BETWEEN_HICCUPS	FI_FPGA_MAIN_REG(0x014C)
#define PCIE_MAX_1S_HICCUP_COUNT	FI_FPGA_MAIN_REG(0x0150)


#define I2C_MASTER_CONTROL		FI_FPGA_MAIN_REG(0x0160)
#define I2C_MASTER_STATUS		FI_FPGA_MAIN_REG(0x0164)
#define I2C_MASTER_CMD_DATA		FI_FPGA_MAIN_REG(0x0168)
#define I2C_MASTER_WR_DATA    		FI_FPGA_MAIN_REG(0x016C)
#define I2C_MASTER_RD_DATA    		FI_FPGA_MAIN_REG(0x0170)

#define I2C_MASTER_CONTROL_ISB		FI_FPGA_MAIN_REG(0x0180)
#define I2C_MASTER_STATUS_ISB		FI_FPGA_MAIN_REG(0x0184)
#define I2C_MASTER_CMD_DATA_ISB		FI_FPGA_MAIN_REG(0x0188)
#define I2C_MASTER_WR_DATA_ISB		FI_FPGA_MAIN_REG(0x018C)
#define I2C_MASTER_RD_DATA_ISB		FI_FPGA_MAIN_REG(0x0190)


#define I2C_MASTER_CONTROL_IOB		FI_FPGA_MAIN_REG(0x01A0)
#define I2C_MASTER_STATUS_IOB		FI_FPGA_MAIN_REG(0x01A4)
#define I2C_MASTER_CMD_DATA_IOB		FI_FPGA_MAIN_REG(0x01A8)
#define I2C_MASTER_WR_DATA_IOB		FI_FPGA_MAIN_REG(0x01AC)
#define I2C_MASTER_RD_DATA_IOB		FI_FPGA_MAIN_REG(0x01B0)

#define POWER_KILL_COMMAND		FI_FPGA_MAIN_REG(0x01BC)

#define LED_CONTROL			FI_FPGA_MAIN_REG(0x01C0)

#define WATCHDOG_CONTROL_STATUS		FI_FPGA_MAIN_REG(0x01F4)
#define WATCHDOG_TIMEOUT_VAL		FI_FPGA_MAIN_REG(0x01F8)
#define WATCHDOG_TICKLE			FI_FPGA_MAIN_REG(0x01FC)

#define PCIE_AVMM_ARB_PORT_B00_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0800)
#define PCIE_AVMM_ARB_PORT_B01_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0804)
#define PCIE_AVMM_ARB_PORT_B02_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0808)
#define PCIE_AVMM_ARB_PORT_B03_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x080C)
#define PCIE_AVMM_ARB_PORT_B04_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0810)
#define PCIE_AVMM_ARB_PORT_B05_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0814)
#define PCIE_AVMM_ARB_PORT_B06_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0818)
#define PCIE_AVMM_ARB_PORT_B07_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x081C)
#define PCIE_AVMM_ARB_PORT_B08_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0820)
#define PCIE_AVMM_ARB_PORT_B09_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0824)
#define PCIE_AVMM_ARB_PORT_B10_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0828)
#define PCIE_AVMM_ARB_PORT_B11_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x082C)
#define PCIE_AVMM_ARB_PORT_B12_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0830)
#define PCIE_AVMM_ARB_PORT_B13_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0834)
#define PCIE_AVMM_ARB_PORT_B14_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0838)
#define PCIE_AVMM_ARB_PORT_B15_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x083C)

#define PCIE_AVMM_ARB_PORT_B00_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0840)
#define PCIE_AVMM_ARB_PORT_B01_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0844)
#define PCIE_AVMM_ARB_PORT_B02_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0848)
#define PCIE_AVMM_ARB_PORT_B03_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x084C)
#define PCIE_AVMM_ARB_PORT_B04_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0850)
#define PCIE_AVMM_ARB_PORT_B05_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0854)
#define PCIE_AVMM_ARB_PORT_B06_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0858)
#define PCIE_AVMM_ARB_PORT_B07_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x085C)
#define PCIE_AVMM_ARB_PORT_B08_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0860)
#define PCIE_AVMM_ARB_PORT_B09_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0864)
#define PCIE_AVMM_ARB_PORT_B10_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0868)
#define PCIE_AVMM_ARB_PORT_B11_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x086C)
#define PCIE_AVMM_ARB_PORT_B12_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0870)
#define PCIE_AVMM_ARB_PORT_B13_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0874)
#define PCIE_AVMM_ARB_PORT_B14_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x0878)
#define PCIE_AVMM_ARB_PORT_B15_THROUGHPUT_COUNT_MAX	FI_FPGA_MAIN_REG(0x087C)

#define PCIE_AVMM_ARB_PORT_B00_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x0880)
#define PCIE_AVMM_ARB_PORT_B01_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x0884)
#define PCIE_AVMM_ARB_PORT_B02_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x0888)
#define PCIE_AVMM_ARB_PORT_B03_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x088C)
#define PCIE_AVMM_ARB_PORT_B04_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x0890)
#define PCIE_AVMM_ARB_PORT_B05_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x0894)
#define PCIE_AVMM_ARB_PORT_B06_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x0898)
#define PCIE_AVMM_ARB_PORT_B07_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x089C)
#define PCIE_AVMM_ARB_PORT_B08_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x08A0)
#define PCIE_AVMM_ARB_PORT_B09_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x08A4)
#define PCIE_AVMM_ARB_PORT_B10_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x08A8)
#define PCIE_AVMM_ARB_PORT_B11_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x08AC)
#define PCIE_AVMM_ARB_PORT_B12_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x08B0)
#define PCIE_AVMM_ARB_PORT_B13_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x08B4)
#define PCIE_AVMM_ARB_PORT_B14_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x08B8)
#define PCIE_AVMM_ARB_PORT_B15_THROUGHPUT_COUNT_MIN	FI_FPGA_MAIN_REG(0x08BC)

#define PCIE_AUX_WRITE_DMA_DATA_FIFO_MAX_LEVEL	FI_FPGA_MAIN_REG(0x08C0)
#define PCIE_AUX_WRITE_DMA_CMD_FIFO_MAX_LEVEL	FI_FPGA_MAIN_REG(0x08C4)
#define PCIE_IRQ_LATENCY_TIMER		FI_FPGA_MAIN_REG(0x08C8)

#define IM_AUX_WR_DMA0_CONTROL		FI_FPGA_MAIN_REG(0x0900)
#define IM_AUX_WR_DMA0_STATUS		FI_FPGA_MAIN_REG(0x0904)
#define IM_AUX_WR_DMA0_START_ADDR	FI_FPGA_MAIN_REG(0x0908)
#define IM_AUX_WR_DMA0_BLOCK_SIZE	FI_FPGA_MAIN_REG(0x090C) // in QWORDS, 4x4=16 bytes each
#define IM_AUX_WR_DMA0_BURST_SIZE	FI_FPGA_MAIN_REG(0x0910) // in QWORDS
#define IM_AUX_WR_DMA0_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0914)
#define IM_AUX_WR_DMA0_BURST_INTERVAL	FI_FPGA_MAIN_REG(0x0918)
#define IM_AUX_WR_DMA0_PATTERN_VAL	FI_FPGA_MAIN_REG(0x091C)

#define IM_AUX_RD_DMA0_CONTROL		FI_FPGA_MAIN_REG(0x0940)
#define IM_AUX_RD_DMA0_STATUS		FI_FPGA_MAIN_REG(0x0944)
#define IM_AUX_RD_DMA0_START_ADDR	FI_FPGA_MAIN_REG(0x0948)
#define IM_AUX_RD_DMA0_BLOCK_SIZE	FI_FPGA_MAIN_REG(0x094C) // in QWORDS, 4x4 words = 16 bytes
#define IM_AUX_RD_DMA0_BURST_SIZE	FI_FPGA_MAIN_REG(0x0950) // in QWORDS
#define IM_AUX_RD_DMA0_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0954)
#define IM_AUX_RD_DMA0_BURST_INTERVAL	FI_FPGA_MAIN_REG(0x0958)
#define IM_AUX_RD_DMA0_PATTERN_VAL	FI_FPGA_MAIN_REG(0x095C)
#define IM_AUX_RD_DMA0_ERROR_COUNT	FI_FPGA_MAIN_REG(0x0960)

#define IM_AUX_WR_DMA1_CONTROL		FI_FPGA_MAIN_REG(0x0980)
#define IM_AUX_WR_DMA1_STATUS		FI_FPGA_MAIN_REG(0x0984)
#define IM_AUX_WR_DMA1_START_ADDR	FI_FPGA_MAIN_REG(0x0988)
#define IM_AUX_WR_DMA1_BLOCK_SIZE	FI_FPGA_MAIN_REG(0x098C) // in QWORDS, 4x4=16 bytes each
#define IM_AUX_WR_DMA1_BURST_SIZE	FI_FPGA_MAIN_REG(0x0990) // in QWORDS
#define IM_AUX_WR_DMA1_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0994)
#define IM_AUX_WR_DMA1_BURST_INTERVAL	FI_FPGA_MAIN_REG(0x0998)
#define IM_AUX_WR_DMA1_PATTERN_VAL	FI_FPGA_MAIN_REG(0x099C)

#define IM_AUX_RD_DMA1_CONTROL		FI_FPGA_MAIN_REG(0x09C0)
#define IM_AUX_RD_DMA1_STATUS		FI_FPGA_MAIN_REG(0x09C4)
#define IM_AUX_RD_DMA1_START_ADDR	FI_FPGA_MAIN_REG(0x09C8)
#define IM_AUX_RD_DMA1_BLOCK_SIZE	FI_FPGA_MAIN_REG(0x09CC) // in QWORDS, 4x4 words = 16 bytes
#define IM_AUX_RD_DMA1_BURST_SIZE	FI_FPGA_MAIN_REG(0x09D0) // in QWORDS
#define IM_AUX_RD_DMA1_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x09D4)
#define IM_AUX_RD_DMA1_BURST_INTERVAL	FI_FPGA_MAIN_REG(0x09D8)
#define IM_AUX_RD_DMA1_PATTERN_VAL	FI_FPGA_MAIN_REG(0x09DC)
#define IM_AUX_RD_DMA1_ERROR_COUNT	FI_FPGA_MAIN_REG(0x09E0)

#define IPM_P1_FRAME_REQ_CONTROL	FI_FPGA_MAIN_REG(0x0A00)
#define IPM_P1_FRAME_REQ_STATUS		FI_FPGA_MAIN_REG(0x0A04)
#define IPM_P1_FRAME_REQ_SRC_PTR	FI_FPGA_MAIN_REG(0x0A08)
#define IPM_P1_FRAME_REQ_DEST_PTR	FI_FPGA_MAIN_REG(0x0A0C)
#define IPM_P1_FRAME_REQ_IC_Y_BUF_START	FI_FPGA_MAIN_REG(0x0A10)
#define IPM_P1_FRAME_REQ_IC_C_BUF_START	FI_FPGA_MAIN_REG(0x0A14)

#define PME_CONTROL			FI_FPGA_MAIN_REG(0x0A40)
#define PME_STATUS			FI_FPGA_MAIN_REG(0x0A44)
#define PME_PARTITION_START		FI_FPGA_MAIN_REG(0x0A48)
#define PME_PARTITION_END		FI_FPGA_MAIN_REG(0x0A4C)
#define PME_BLOCK_START			FI_FPGA_MAIN_REG(0x0A50)
#define PME_BLOCK_END			FI_FPGA_MAIN_REG(0x0A54)
#define PME_FRAME_BUF_SIZE		FI_FPGA_MAIN_REG(0x0A58)
#define PME_FRAME_COUNT			FI_FPGA_MAIN_REG(0x0A5C)
#define PME_METADATA_MWORD_OFFSET	FI_FPGA_MAIN_REG(0x0A60)
#define PME_OUTPUT_BUF_START_PTR	FI_FPGA_MAIN_REG(0x0A64)

#define IM2PCIE_DMA_CONTROL		FI_FPGA_MAIN_REG(0x0C00)
#define IM2PCIE_DMA_STATUS		FI_FPGA_MAIN_REG(0x0C04)
#define IM2PCIE_IM_RD_DMA_START		FI_FPGA_MAIN_REG(0x0C08)
#define IM2PCIE_IM_RD_DMA_BLOCK_SIZE	FI_FPGA_MAIN_REG(0x0C0C)
#define IM2PCIE_IM_RD_DMA_BURST_SIZE	FI_FPGA_MAIN_REG(0x0C10)
#define IM2PCIE_IM_RD_DMA_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0C14)
#define IM2PCIE_PCIE_WR_DMA_START	FI_FPGA_MAIN_REG(0x0C18)
#define IM2PCIE_PCIE_WR_DMA_BLOCK_SIZE	FI_FPGA_MAIN_REG(0x0C1C)
#define IM2PCIE_PCIE_WR_DMA_BURST_SIZE	FI_FPGA_MAIN_REG(0x0C20)
#define IM2PCIE_PCIE_WR_DMA_BURST_PERIOD	FI_FPGA_MAIN_REG(0x0C24)
#define IM2PCIE_PCIE_WR_DMA_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0C28)
#define IM2PCIE_DMA_FRAME_DIMENSIONS	FI_FPGA_MAIN_REG(0x0C2C)

#define IM2SSD_DMA0_CONTROL		FI_FPGA_MAIN_REG(0x0C40)
#define IM2SSD_DMA0_STATUS		FI_FPGA_MAIN_REG(0x0C44)
#define IM2SSD_DMA0_IM_RD_DMA_BLOCK_SIZE	FI_FPGA_MAIN_REG(0x0C48)
#define IM2SSD_DMA0_IM_RD_DMA_BURST_SIZE	FI_FPGA_MAIN_REG(0x0C4C)
#define IM2SSD_DMA0_IM_RD_DMA_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0C50)
#define IM2SSD_DMA0_LR_WR_FRAME_COUNT_LATCHED	FI_FPGA_MAIN_REG(0x0C54)
#define IM2SSD_DMA0_IM_RD_DMA_START_ADDRESS	FI_FPGA_MAIN_REG(0x0C58)
#define IM2SSD_DMA0_LR_DESCR_FIFO_STATUS	FI_FPGA_MAIN_REG(0x0C60)
#define IM2SSD_DMA0_LR_DESCR_FIFO_AFULL_LEVEL	FI_FPGA_MAIN_REG(0x0C64)
#define IM2SSD_DMA0_PCIE_BURST_SIZE	FI_FPGA_MAIN_REG(0x0C68)
#define IM2SSD_DMA0_PCIE_BURST_TIMER	FI_FPGA_MAIN_REG(0x0C6C)
#define IM2SSD_DMA0_PADDING_INTERVAL	FI_FPGA_MAIN_REG(0x0C70)
#define IM2SSD_DMA0_PADDING_QWORD_COUNT	FI_FPGA_MAIN_REG(0x0C74)

#define PCIE2IM_DMA_CONTROL		FI_FPGA_MAIN_REG(0x0C80)
#define PCIE2IM_DMA_STATUS		FI_FPGA_MAIN_REG(0x0C84)
#define PCIE2IM_PCIE_RD_DMA_START	FI_FPGA_MAIN_REG(0x0C88)
#define PCIE2IM_PCIE_RD_DMA_BLOCK_SIZE	FI_FPGA_MAIN_REG(0x0C8C)
#define PCIE2IM_PCIE_RD_DMA_BURST_SIZE	FI_FPGA_MAIN_REG(0x0C90)
#define PCIE2IM_PCIE_RD_DMA_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0C94)
#define PCIE2IM_IM_WR_DMA_START	FI_FPGA_MAIN_REG(0x0C98)
#define PCIE2IM_IM_WR_DMA_BURST_SIZE	FI_FPGA_MAIN_REG(0x0C9C)
#define PCIE2IM_IM_WR_DMA_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0CA0)

#define SSD2IM_DMA_CONTROL		FI_FPGA_MAIN_REG(0x0CC0)
#define SSD2IM_DMA_STATUS		FI_FPGA_MAIN_REG(0x0CC4)
#define SSD2IM_DMA_BLOCK_SIZE		FI_FPGA_MAIN_REG(0x0CC8)
#define SSD2IM_DMA_IM_WR_DMA_START	FI_FPGA_MAIN_REG(0x0CCC)
#define SSD2IM_DMA_IM_WR_DMA_BURST_SIZE	FI_FPGA_MAIN_REG(0x0CD0)
#define SSD2IM_DMA_IM_WR_DMA_BP_THRESHOLDS	FI_FPGA_MAIN_REG(0x0CD4)
#define SSD2IM_DMA_PCIE_BURST_SIZE	FI_FPGA_MAIN_REG(0x0CD8)
#define SSD2IM_DMA_PCIE_BURST_TIMER	FI_FPGA_MAIN_REG(0x0CDC)

#define IM2PCIE_DMA_BUSY_COUNT_LAST	FI_FPGA_MAIN_REG(0x0E00)
#define IM2PCIE_DMA_BUSY_COUNT_MAX 	FI_FPGA_MAIN_REG(0x0E04)
#define IM2PCIE_DMA_BUSY_COUNT_MIN 	FI_FPGA_MAIN_REG(0x0E08)
#define IM2PCIE_DMA_IDLE_COUNT_LAST	FI_FPGA_MAIN_REG(0x0E0C)
#define IM2PCIE_DMA_IDLE_COUNT_MAX 	FI_FPGA_MAIN_REG(0x0E10)
#define IM2PCIE_DMA_IDLE_COUNT_MIN 	FI_FPGA_MAIN_REG(0x0E14)

//-------------------------------------
// NC1_FPGA_CONTROL bits
//-------------------------------------
#define NC1FC_EMIF_SW_RESET		(1 << 0)
#define NC1FC_LED_A10_STAT0		(1 << 4)
#define NC1FC_LED_A10_STAT1		(1 << 5)
#define NC1FC_LED_TEMP_FAULT		(1 << 6)
#define NC1FC_ISB_PWR_ENABLE		(1 << 8)
#define NC1FC_SHUTTER_ENABLE		(1 << 12)
#define NC1FC_SW_FLAGS			(0xF << 20)
#define NC1FC_SW_COLD_BOOT		(0x0 << 20)
#define NC1FC_SW_ACTIVE			(0xA << 20)
#define NC1FC_FAN_PWM_TEST		((uint32_t)(1 << 31))

//-------------------------------------
// NC1_FPGA_STATUS bits
//-------------------------------------
#define NC1FS_EMIF_LOCAL_CAL_SUCCESS	(1 << 0)
#define NC1FS_I2C_ENABLED		(1 << 4)
#define NC1FS_ISB_PWR_OK		(1 << 8)

//-------------------------------------
// PCIE_IRQ_STATUS bits
//-------------------------------------
#define PCIEIRQ_FIFO_NOT_EMPTY		(1 << 0)
#define PCIEIRQ_P1_FRAME_DONE		(1 << 2)
#define PCIEIRQ_IM2PCIE_DMA_DONE	(1 << 4)
#define PCIEIRQ_PCIE2IM_DMA_DONE	(1 << 5)
#define PCIEIRQ_EXT_ARM_INPUT           (1 << 6)
#define PCIEIRQ_EXT_UNARM_INPUT         (1 << 7)
#define PCIEIRQ_LR_DESC_FIFO_AF		(1 << 8)
#define PCIEIRQ_MFT_LSDET		(1 << 12)
#define PCIEIRQ_MFT_LSLPSW		(1 << 13)
#define PCIEIRQ_PWR_OFF			(1 << 30)
#define PCIEIRQ_TEST			((uint32_t)(1 << 31))

//-------------------------------------
// I2C_MASTER_CONTROL bits
//-------------------------------------
#define I2CM_CMD_DEV_ADDR_SHIFT		12
#define I2CM_CMD_DEV_ADDR_MASK		(0xFF << I2CM_CMD_DEV_ADDR_SHIFT)
#define I2CM_CMD_DEV(x)			((x) << I2CM_CMD_DEV_ADDR_SHIFT)

#define I2CM_CMD_1CMD_CNT		(1 << 8)
#define I2CM_CMD_4CMD_CNT		(4 << 8)

#define I2CM_CMD_NO_DATA		(0 << 4)
#define I2CM_CMD_1DATA_CNT		(1 << 4)
#define I2CM_CMD_2DATA_CNT		(2 << 4)
#define I2CM_CMD_4DATA_CNT		(4 << 4)

#define I2CM_CMD_READ_ONLY		(2 << 0)
#define I2CM_CMD_READ			(1 << 0)
#define I2CM_CMD_WRITE			(0 << 0)

//-------------------------------------
// I2C_MASTER_STATUS bits
//-------------------------------------
#define I2CM_STATUS_BUSY		((uint32_t)(1<<31))
#define I2CM_STATUS_ERR			((uint32_t)(1<<30))

//-------------------------------------
// POWER_KILL_COMMAND bits
//-------------------------------------
#define POWER_KILL_ENABLE		((uint32_t)0x3210ABCD)

//-------------------------------------
// LED_CONTROL bits
//-------------------------------------
#define LC_LED_MODE_OFF			0
#define LC_LED_MODE_SOLID_ON		1
#define LC_LED_MODE_BLINK		2
#define LC_LED_MODE_MASK		0x3

#define LC_LED_BLINK_COUNT_SHIFT	4
#define LC_LED_BLINK_COUNT_MASK		(0x7 << LC_LED_BLINK_COUNT_SHIFT)
#define LC_LED_BLINK_COUNT(x)		(((x) << LC_LED_BLINK_COUNT_SHIFT) & LC_LED_BLINK_COUNT_MASK)

#define LC_LED_BLINK_ONTIME_SHIFT	8
#define LC_LED_BLINK_ONTIME_MASK	(0x3F << LC_LED_BLINK_ONTIME_SHIFT)
#define LC_LED_BLINK_ONTIME(x)		(((x) << LC_LED_BLINK_ONTIME_SHIFT) & LC_LED_BLINK_ONTIME_MASK)

#define LC_LED_BLINK_OFFTIME_SHIFT	16
#define LC_LED_BLINK_OFFTIME_MASK	(0x3F << LC_LED_BLINK_OFFTIME_SHIFT)
#define LC_LED_BLINK_OFFTIME(x)		(((x) << LC_LED_BLINK_OFFTIME_SHIFT) & LC_LED_BLINK_OFFTIME_MASK)

#define LC_LED_BLINK_GAPTIME_SHIFT	16
#define LC_LED_BLINK_GAPTIME_MASK	(0x3F << LC_LED_BLINK_GAPTIME_SHIFT)
#define LC_LED_BLINK_GAPTIME(x)		(((x) << LC_LED_BLINK_GAPTIME_SHIFT) & LC_LED_BLINK_GAPTIME_MASK)

//-------------------------------------
// WATCHDOG bits
//-------------------------------------
#define WD_ENABLE			(1 << 0)
#define WD_EXPIRED			((uint32_t)(1 << 31))
#define WD_TICKLE_MAGIC			((uint32_t)0xABADBABE)

//-------------------------------------
// AUX_RD/AUX_WR_DMA_STATUS bits
//-------------------------------------
#define AUX_DMA_BUSY			(1 << 0)

//-------------------------------------
// AUX_RD/AUX_WR_DMA_CONTROL bits
//-------------------------------------
#define AUX_DMA_ENABLE			(1 << 0)
#define AUX_DMA_START			(1 << 1)
#define AUX_DMA_AUTO_REPEAT		(1 << 2)
#define AUX_DMA_REQ_TIMER_ENABLE	(1 << 3)
#define AUX_DMA_PATTERN_MASK		(3 << 4)
#define AUX_DMA_PATTERN_SHIFT		 4
#define AUX_DMA_PATTERN_VAL		    (0 << 4)
#define AUX_DMA_PATTERN_INCREMENTING	    (1 << 4)
#define AUX_DMA_PATTERN_ADDRESS		    (2 << 4)
#define AUX_DMA_PATTERN_RANDOM		    (3 << 4)
#define AUX_DMA_RD_ERROR_AUTOCLEAR	(1 << 6)	// AUX_RD_DMA_CONTROL only
#define AUX_DMA_BYTE_ORDER_R2L		(1 << 8)
#define AUX_DMA_WORD_ORDER_R2L		(1 << 9)

//-------------------------------------
// TIME_CONTROL register bit definitions
//-------------------------------------
#define TIME_MODE_MASTER		0
#define TIME_MODE_1PPS_SLAVE		1
#define TIME_MODE_IRIG_SLAVE		2
#define TIME_MODE_MASK			0x03
#define TIME_MODE_SET			(1 << 2)
#define TIME_STAMP_SNAPSHOT		(1 << 4)
#define TIME_SYNC_LATE_SHIFT		8
#define TIME_SYNC_LATE_MASK		( 0xFF < TIME_SYNC_LATE_SHIFT )
#define TIME_SYNC_LATE_LIMIT(x)		((x) << TIME_SYNC_LATE_SHIFT )

//-------------------------------------
// TIME_STATUS register bit definitions
//-------------------------------------
#define TIME_STATUS_STATE(x)		((x) & 0x07)
#define TIME_STATUS_SYNC_LATE		(1 << 4)

//-------------------------------------
// IPM_P1_FRAME_REQ_CONTROL register bit definitions
//-------------------------------------
#define IPM_P1_FRAME_REQUEST		(1 << 0)
#define IPM_P1_IC_ENABLE		(1 << 1)
#define IPM_P1_DMA_ENABLE		(1 << 2)

//-------------------------------------
// IPM_P1_FRAME_REQ_STATUS register bit definitions
//-------------------------------------
#define IPM_P1_FRAME_BUSY		(1 << 0)

//-------------------------------------
// PME_CONTROL register definitions
//-------------------------------------
#define PMEC_START			(1 << 0)
#define PMEC_ABORT			(1 << 4)

//-------------------------------------
// PME_STATUS register definitions
//-------------------------------------
#define PMES_WR_DMA_BUSY		(1 << 0)
#define PMES_BUSY			(1 << 1)

//-------------------------------------
// IM2PCIE_DMA_CONTROL register bit definitions
//-------------------------------------
#define IM2PCIE_DMA_ENABLE		(1 << 0)
#define IM2PCIE_DMA_START		(1 << 1)
#define IM2PCIE_DMA_BURST_TIMER_EN 	(1 << 3)
#define IM2PCIE_DMA_GENERIC_DATA	(0 << 4)
#define IM2PCIE_DMA_IMAGE_DATA		(1 << 4)
#define IM2PCIE_DMA_BYTE_ORDER_L2R	(0 << 5)
#define IM2PCIE_DMA_BYTE_ORDER_R2L	(1 << 5)
#define IM2PCIE_DMA_NO_ALIGN		(0 << 6)
#define IM2PCIE_DMA_WORD_ALIGN		(1 << 6)
#define IM2PCIE_DMA_WORD_ORDER_L2R	(0 << 7)
#define IM2PCIE_DMA_WORD_ORDER_R2L	(1 << 7)
#define IM2PCIE_DMA_FORMAT_SHIFT	8
#define IM2PCIE_DMA_FORMAT_MASK		(7 << 8)
#define IM2PCIE_DMA_RGB_FORMAT			(0 << 8)	/* Image is native IPM format: RGB */
#define IM2PCIE_DMA_BGR_FORMAT			(1 << 8)	/* Swap image from RGB to BGR */
#define IM2PCIE_DMA_8BIT_RAW_FMT		(2 << 8)	/* 8 bit data */
#define IM2PCIE_DMA_10BIT_RAW_TO_16		(3 << 8)	/* Expand 10 bits to 16 bits */
#define IM2PCIE_DMA_12BIT_RAW_TO_16		(4 << 8)	/* Expand 12 bits to 16 bits */
#define IM2PCIE_DMA_16BIT_RAW_FMT		(5 << 8)	/* The IPM already expanded pixels to 16 bits */
#define IM2PCIE_DMA_PADDING_SHIFT	16
#define IM2PCIE_DMA_PADDING_MASK	(0xFF << 16)
#define IM2PCIE_DMA_HIGHSPEED_BYPASS	(1 << 24)

//-------------------------------------
// IM2PCIE_DMA_STATUS register bit definitions
//-------------------------------------
#define IM2PCIE_DMA_BUSY		(1 << 0)

//-------------------------------------
// IM2PCIE_xxx_DMA_START_ADDR register bit definitions
// IM: 64-byte MWORD address
// PC: 16-byte QWORD address
//-------------------------------------
#define IM2PCIE_DMA_START_ADDR_MASK	0x3FFFFFFF

//-------------------------------------
// IM2PCIE_xxx_DMA_BLOCK_SIZE register bit definitions
// IM: 64-byte MWORDs
// PC: 16-byte QWORDs
//-------------------------------------
#define IM2PCIE_DMA_BLOCK_SIZE_MASK	0x1FFFFFFF

//-------------------------------------
// IM2PCIE_RD_DMA_BURST_SIZE register bit definitions
// IM: 64-byte MWORDs
// PC: 16-byte QWORDs
//-------------------------------------
#define IM2PCIE_RD_DMA_BURST_SIZE_MASK	0x0000007F

//-------------------------------------
// IM2PCIE_IM_RD_DMA_BP_THRESHOLDS register bit definitions
//-------------------------------------
#define IM2PCIE_DMA_BP_CMD_AFULL( x )	((x) & 0xFF)
#define IM2PCIE_DMA_BP_MAX_PENDING( x )	(((x) & 0xFF) << 8)

//-------------------------------------
// IM2PCIE_WR_DMA_BURST_SIZE register bit definitions
//-------------------------------------
#define IM2PCIE_WR_DMA_BURST_SIZE_MASK	0x0000003F

//-------------------------------------
// IM2PCIE_WR_DMA_BURST_PERIOD register bit definitions
// in 200MHz clock cycles
//-------------------------------------
#define IM2PCIE_WR_DMA_BURST_PERIOD_MASK	0x0000FFFF

//-------------------------------------
// IM2PCIE_WR_DMA_BP_THRESHOLDS register bit definitions
//-------------------------------------
#define IM2PCIE_WR_DMA_BP_CMD_FIFO( x )		((x) & 0xFF)
#define IM2PCIE_WR_DMA_BP_DATA_FIFO( x )	(((x) & 0xFF) << 8)

//-------------------------------------
// IM2PCIE_DMA_FRAME_DIMENSIONS register bit definitions
//-------------------------------------
#define IM2PCIE_DMA_FRAME_X(x)		((x) & 0x01FFF)
#define IM2PCIE_DMA_FRAME_Y(x)		(((x) & 0x01FFF) << 16)
#define IM2PCIE_GET_DMA_FRAME_Y(x)      (((x) >> 16 ) & 0x1FFF)

//-------------------------------------
// IM2SSD_DMA0_CONTROL register bit definitions
//-------------------------------------
#define IM2SSD_DMA0_ENABLE		(1 << 0)
#define IM2SSD_DMA0_START		(1 << 1)
#define IM2SSD_DMA0_LR_MODE_ENABLE	(1 << 4)
#define IM2SSD_DMA0_FIFO_EMPTY_OR	(1 << 5)
#define IM2SSD_DMA0_PADDING_ENABLE	(1 << 6)
#define IM2SSD_DMA0_BURST_TIMER_ARM	(1 << 8)
#define IM2SSD_DMA0_BURST_TIMER_CLR	(1 << 9)
#define IM2SSD_DMA0_WORD_ORDER_L2R	(0 << 12)
#define IM2SSD_DMA0_WORD_ORDER_R2L	(1 << 12)
#define IM2SSD_DMA0_BYTE_ORDER_L2R	(0 << 13)
#define IM2SSD_DMA0_BYTE_ORDER_R2L	(1 << 13)

//-------------------------------------
// IM2SSD_DMA0_STATUS register bit definitions
//-------------------------------------
#define IM2SSD_DMA0_BUSY		(1 << 0)
#define IM2SSD_DMA0_LR_IDLE		(1 << 1)
#define IM2SSD_DMA0_LR_CAPTURE_COMPLETE	(1 << 2)

//-------------------------------------
// IM2SSD_DMA0_LR_DESCR_FIFO_STATUS register bit definitions
//-------------------------------------

// Check to see if the current level indicates an overflow.
#define IM2SSD_DMA0_LR_DESCR_FIFO_OVFLOW	0x1000
#define IM2SSD_DMA0_LR_DESCR_LEVEL_MASK		0xFFF
#define IM2SSD_DMA0_LR_DESCR_LEVEL_MAX_SHIFT	16
#define IM2SSD_DMA0_LR_DESCR_LEVEL_MAX_MASK	(0xFFF << IM2SSD_DMA0_LR_DESCR_LEVEL_MAX_SHIFT)

//-------------------------------------
// IM2SSD_DMA0_PCIE_BURST_TIMER register bit definitions
//-------------------------------------
#define IM2SSD_DMA0_PCIE_BT_READY	((uint32_t)(1 << 31))
#define IM2SSD_DMA0_PCIE_BT_MASK	0x00FFFFFF

//-------------------------------------
// PCIE2IM_DMA_CONTROL register bit definitions
//-------------------------------------
//
// Byte swapping bits work like this:
//
// PCIE2IM_DMA_LITTLE_ENDIAN	// 0x000: BYTE SWAP, WORD ORDER OK.
// 	src: 0x49674867 0x46694669 0x44694a67 0x4766476a
// 	dst: 0x67486749 0x69466946 0x674a6944 0x6a476647
//
// PCIE2IM_DMA_BIG_ENDIAN;	// 0x100: BYTE NO-SWAP, WORD SWAP.
// 	src: 0x49674867 0x46694669 0x44694a67 0x4766476a
// 	dst: 0x4766476a 0x44694a67 0x46694669 0x49674867
//
// PCIE2IM_DMA_WORD_ORDER_R2L;	// 0x200: BYTE NO-SWAP, WORD ORDER OK.
// 	src: 0x49674867 0x46694669 0x44694a67 0x4766476a
// 	dst: 0x49674867 0x46694669 0x44694a67 0x4766476a
//
// PCIE2IM_DMA_BIG_ENDIAN | PCIE2IM_DMA_WORD_ORDER_R2L;	// 0x300: BYTE SWAP, WORD SWAP.
//	src: 0x49674867 0x46694669 0x44694a67 0x4766476a
//	dst: 0x6a476647 0x674a6944 0x69466946 0x67486749
#define PCIE2IM_DMA_ENABLE		(1 << 0)
#define PCIE2IM_DMA_START		(1 << 1)    /* write-only, self-clears */
#define PCIE2IM_DMA_BYTE_ORDER_L2R	(0 << 8)
#define PCIE2IM_DMA_BYTE_ORDER_R2L	(1 << 8)
#define PCIE2IM_DMA_WORD_ORDER_L2R	(0 << 9)
#define PCIE2IM_DMA_WORD_ORDER_R2L	(1 << 9)

//-------------------------------------
// PCIE2IM_DMA_STATUS register bit definitions
//-------------------------------------
#define PCIE2IM_DMA_BUSY		(1 << 0)

//-------------------------------------
// SSD2IM_DMA_CONTROL register bit definitions
//-------------------------------------
#define SSD2IM_DMA_ENABLE		(1 << 0)
#define SSD2IM_DMA_START		(1 << 1)
#define SSD2IM_DMA_BURST_TIMER_ARM	(1 << 8)
#define SSD2IM_DMA_BURST_TIMER_CLR	(1 << 9)
#define SSD2IM_DMA_WORD_ORDER_L2R	(0 << 12)
#define SSD2IM_DMA_WORD_ORDER_R2L	(1 << 12)
#define SSD2IM_DMA_BYTE_ORDER_L2R	(0 << 13)
#define SSD2IM_DMA_BYTE_ORDER_R2L	(1 << 13)

//-------------------------------------
// SSD2IM_DMA_STATUS register bit definitions
//-------------------------------------
#define SSD2IM_DMA_BUSY			(1 << 0)

//-------------------------------------
// SSD2IM_DMA_PCIE_BURST_TIMER register bit definitions
//-------------------------------------
#define SSD2IM_DMA_PCIE_BT_READY	((uint32_t)(1 << 31))
#define SSD2IM_DMA_PCIE_BT_MASK		0x00FFFFFF


//-------------------------------------
// EXT_IO_CONTROL register bit definitions
//-------------------------------------

#define EXTIO_TRIGIN_RISING_EDGE	(1 << 0)
#define EXTIO_TRIGIN_FALLING_EDGE	0
#define EXTIO_SYNCIN_RISING_EDGE	(1 << 1)
#define EXTIO_SYNCIN_FALLING_EDGE	0
#define EXTIO_ARMIN_RISING_EDGE		(1 << 2)
#define EXTIO_ARMIN_FALLING_EDGE	0
#define EXTIO_ARMIN_LEVEL_INPUT		(1 << 3)
#define EXTIO_ARMIN_EDGE_INPUT		0
#define EXTIO_TRIGOUT_ENABLE     	(1 << 4)
#define EXTIO_SYNCOUT_ENABLE		(1 << 5)
#define EXTIO_ARMOUT_ENABLE		(1 << 6)
// Reserved bit[7]
#define EXTIO_TRIGOUT_PASSTHRU_ENABLE	(1 << 8)
#define EXTIO_SYNCOUT_PASSTHRU_ENABLE	(1 << 9)
#define EXTIO_ARMOUT_PASSTHRU_ENABLE	(1 << 10)
// Reserved bit[11]
//
// For the following, if the source of the output pin is the
// internal signal, then a '0' means the output will be active low
// and a '1' means the output will be active high. If the output
// is a "pass-through", then a '0' passes the signal straight
// through and a '1' will invert the signal. Thus the multiple
// defines for the same pin.
#define EXTIO_TRIGOUT_ACTIVE_HIGH	(1 << 12)
#define EXTIO_TRIGOUT_INVERT		(1 << 12)
#define EXTIO_TRIGOUT_ACTIVE_LOW	0
#define EXTIO_TRIGOUT_PASS_THRU		0
#define EXTIO_SYNCOUT_ACTIVE_HIGH	(1 << 13)
#define EXTIO_SYNCOUT_INVERT		(1 << 13)
#define EXTIO_SYNCOUT_ACTIVE_LOW	0
#define EXTIO_SYNCOUT_PASS_THRU		0
#define EXTIO_ARMOUT_ACTIVE_HIGH	(1 << 14)
#define EXTIO_ARMOUT_INVERT		(1 << 14)
#define EXTIO_ARMOUT_ACTIVE_LOW		0
#define EXTIO_ARMOUT_PASS_THRU		0
// Reserved bit[15]
#define EXTIO_SYNCOUT_SRC_SEL_MASK	(3 << 16)
#define EXTIO_SYNCOUT_INTERNAL_1PPF		(0 << 16)
#define EXTIO_SYNCOUT_PROG_1PPF			(2 << 16)
#define EXTIO_SYNCOUT_PROG_1PPS			(3 << 16)
#define EXTIO_SYNC_1PPS_SRC_SEL_MASK	(3 << 18)
#define EXTIO_SYNC_1PPS_SYNC_IN			(0 << 18)
#define EXTIO_SYNC_1PPS_INTERNAL		(1 << 18)
#define EXTIO_SYNC_1PPS_GPS			(2 << 18)
#define EXTIO_SYNC_1PPS_IRIG			(3 << 18)
#define EXTIO_ARM_OUT_PROG_PULSE	(1 << 20)
#define EXTIO_ARM_OUT_INTERNAL_STATE	0
// Reserved bit[21]
#define EXTIO_SYNC_PERIOD_MEAS_MASK	(3 << 22)
#define EXTIO_SYNC_PERIOD_MEAS_EXT_SYNC		(0 << 22)
#define EXTIO_SYNC_PERIOD_MEAS_INT_1PPS		(1 << 22)
#define EXTIO_SYNC_PERIOD_MEAS_GPS		(2 << 22)
#define EXTIO_SYNC_PERIOD_MEAS_IRIG		(3 << 22)
#define EXTIO_TRIGIN_ENABLE		(1 << 24)
#define EXTIO_SYNCIN_ENABLE		(1 << 25)
#define EXTIO_TRIGBTN_IN_ENABLE		(1 << 26)
#define EXTIO_ARM_IN_ENABLE		(1 << 27)
#define EXTIO_TRIG_OUT_TEST		(1 << 28)
#define EXTIO_SYNC_OUT_TEST		(1 << 29)
#define EXTIO_ARM_OUT_TEST		(1 << 30)

//-------------------------------------
// EXT_IO_STATUS register bit definitions
//-------------------------------------
#define EXTST_CHANNEL_0			(1 << 0)
#define EXTST_CHANNEL_1			(1 << 1)
#define EXTST_CHANNEL_2			(1 << 2)

//-------------------------------------
// SYNC_IN_PERIOD_MIN
//-------------------------------------
#define SYNCIN_ACTIVE			((uint32_t)(1 << 31))
#define SYNCIN_PERIOD_MIN_MASK		((1 << 28) - 1)

//-------------------------------------
// EXT_IO_MUX_CONTROL
//-------------------------------------
#define EXTIO_MUX_IO0_SEL_SYNC_OUT	(0 << 0)
#define EXTIO_MUX_IO0_SEL_TRIG_OUT	(1 << 0)
#define EXTIO_MUX_IO0_SEL_ARM_OUT	(2 << 0)
#define EXTIO_MUX_IO0_SEL_DISABLED	(3 << 0)
#define EXTIO_MUX_IO0_DIR_OUTPUT	(1 << 2)
#define EXTIO_MUX_IO1_SEL_SYNC_OUT	(0 << 4)
#define EXTIO_MUX_IO1_SEL_TRIG_OUT	(1 << 4)
#define EXTIO_MUX_IO1_SEL_ARM_OUT	(2 << 4)
#define EXTIO_MUX_IO1_SEL_DISABLED	(3 << 4)
#define EXTIO_MUX_IO1_DIR_OUTPUT	(1 << 6)
#define EXTIO_MUX_IO2_SEL_SYNC_OUT	(0 << 8)
#define EXTIO_MUX_IO2_SEL_TRIG_OUT	(1 << 8)
#define EXTIO_MUX_IO2_SEL_ARM_OUT	(2 << 8)
#define EXTIO_MUX_IO2_SEL_DISABLED	(3 << 8)
#define EXTIO_MUX_IO2_DIR_OUTPUT	(1 << 10)
#define EXTIO_MUX_SYNC_IN_SEL_IO0	(0 << 12)
#define EXTIO_MUX_SYNC_IN_SEL_IO1	(1 << 12)
#define EXTIO_MUX_SYNC_IN_SEL_IO2	(2 << 12)
#define EXTIO_MUX_SYNC_IN_SEL_OFF	(3 << 12)
#define EXTIO_MUX_TRIG_IN_SEL_IO0	(0 << 14)
#define EXTIO_MUX_TRIG_IN_SEL_IO1	(1 << 14)
#define EXTIO_MUX_TRIG_IN_SEL_IO2	(2 << 14)
#define EXTIO_MUX_TRIG_IN_SEL_OFF	(3 << 14)
#define EXTIO_MUX_ARM_IN_SEL_IO0	(0 << 16)
#define EXTIO_MUX_ARM_IN_SEL_IO1	(1 << 16)
#define EXTIO_MUX_ARM_IN_SEL_IO2	(2 << 16)
#define EXTIO_MUX_ARM_IN_SEL_OFF	(3 << 16)
#define EXTIO_MUX_IOB_EXTIO_VSEL_3_3V	(1 << 20)
#define EXTIO_MUX_IOB_EXTIO_VSEL_5V	(0 << 20)

// ======================================================================================

//-------------------------------------
// ASMI Register Space
//-------------------------------------
#define FI_FPGA_ASMI_REG(x) (FI_FPGA_BASE_ADDR + FI_FPGA_ASMI_OFFSET + (x))

#define ASMI_COMMAND			FI_FPGA_ASMI_REG(0x0000)
#define ASMI_STATUS			FI_FPGA_ASMI_REG(0x0004)
#define ASMI_RD_WR_ADDRESS		FI_FPGA_ASMI_REG(0x0008)
#define ASMI_RD_WR_BYTE_COUNT		FI_FPGA_ASMI_REG(0x000C)
#define ASMI_SECTOR_PROTECT_CODE	FI_FPGA_ASMI_REG(0x0010)
#define ASMI_WR_FIFO_DATA		FI_FPGA_ASMI_REG(0x0014)
#define ASMI_WR_FIFO_LEVEL		FI_FPGA_ASMI_REG(0x0018)
#define ASMI_RD_FIFO_DATA		FI_FPGA_ASMI_REG(0x001C)
#define ASMI_RD_FIFO_LEVEL		FI_FPGA_ASMI_REG(0x0020)

#define RU_COMMAND			FI_FPGA_ASMI_REG(0x0040)
#define RU_STATUS			FI_FPGA_ASMI_REG(0x0044)
#define RU_STATUS_READ			FI_FPGA_ASMI_REG(0x0048)
#define RU_CONTROL_READ_A		FI_FPGA_ASMI_REG(0x004C)
#define RU_CONTROL_READ_B		FI_FPGA_ASMI_REG(0x0050)
#define RU_UPDATE_READ_A		FI_FPGA_ASMI_REG(0x0054)
#define RU_UPDATE_READ_B		FI_FPGA_ASMI_REG(0x0058)
#define RU_UPDATE_WRITE_A		FI_FPGA_ASMI_REG(0x005C)
#define RU_UPDATE_WRITE_B		FI_FPGA_ASMI_REG(0x0060)

//-------------------------------------
// ASMI_COMMAND register bit definitions
//-------------------------------------
#define ASMI_EN4B_ENABLE		(1 << 0)
#define ASMI_READ_RDID			(1 << 1)
#define ASMI_READ_STATUS		(1 << 2)
#define ASMI_WRITE			(1 << 3)
#define ASMI_READ			(1 << 4)
#define ASMI_SECTOR_PROTECT		(1 << 5)
#define ASMI_SECTOR_ERASE		(1 << 6)
#define ASMI_DIE_ERASE			(1 << 7)
#define ASMI_BYTE_ORDER_REVERSE		(1 << 8)
#define ASMI_BIT_ORDER_REVERSE		(1 << 9)
#define ASMI_RESET			(1 << 12)

//-------------------------------------
// ASMI_STATUS register bit definitions
//-------------------------------------
#define ASMI_BUSY			(1 << 0)
#define ASMI_ILLEGAL_ERASE_ERR		(1 << 1)
#define ASMI_ILLEGAL_WRITE_ERR		(1 << 2)
#define ASMI_READ_RDID_SHIFT		8
#define ASMI_READ_RDID_MASK		(0xFF << ASMI_READ_RDID_SHIFT)
#define ASMI_READ_STATUS_SHIFT		16
#define ASMI_READ_STATUS_MASK		(0xFF << ASMI_READ_STATUS_SHIFT)

//-------------------------------------
// RU_COMMAND register bit definitions
//-------------------------------------
#define RU_READ_STATUS			(1 << 0)
#define RU_READ_CONTROL			(1 << 1)
#define RU_READ_UPDATE			(1 << 2)
#define RU_WRITE_UPDATE			(1 << 3)
#define RU_RECONFIG			(1 << 4)

//-------------------------------------
// RU_STATUS register bit definitions
//-------------------------------------
#define RU_CONTROLLER_BUSY		(1 << 0)

//-------------------------------------
// RU_STATUS_READ register bit definitions
//-------------------------------------
#define RU_STATUS_READ_MASK		(0x1F)
#define RUS_CRCERROR			(1 << 0)
#define RUS_NSTATUS			(1 << 1)		/* Configuration failed */
#define RUS_RUNCONFIG			(1 << 2)		/* Runtime command from FPGA */
#define RUS_NCONFIG			(1 << 3)		/* External config */
#define RUS_WDTIMER			(1 << 4)

//-------------------------------------
// RU_CONTROL_READ_B, RU_UPDATE_READ_B and RU_UPDATE_WRITE_B register bit definitions
//-------------------------------------
#define RU_WD_TIMER_MASK		(0xFFF)
#define RU_WD_ENABLE			(1 << 12)
#define RU_ANF				(1 << 13)

// ======================================================================================

//-------------------------------------
// MFT Register Space
//-------------------------------------
#define FI_FPGA_MFT_REG(x) (FI_FPGA_BASE_ADDR + FI_FPGA_ASMI_OFFSET + (x))

#define MFT_CONTROL_STATUS		FI_FPGA_MFT_REG(0x0080)
#define MFT_SPI_MASTER_CONFIG		FI_FPGA_MFT_REG(0x0084)
#define MFT_SPI_MASTER_COMMAND		FI_FPGA_MFT_REG(0x0088)
#define MFT_SPI_MASTER_STATUS		FI_FPGA_MFT_REG(0x008C)

//-------------------------------------
// MFT_CONTROL_STATUS register bit definitions
//-------------------------------------
#define MFT_LSPWR_ENABLE		(1 << 0)
#define MFT_LSRST_NOT			(1 << 1)
#define MFT_LSB2L			(1 << 4)
#define MFT_LSLDP			(1 << 5)
#define MFT_LSB2A_LSEXP			(1 << 6)
#define MFT_LSDET_NOT_PRESENT		(1 << 8)
#define MFT_LSLPSW			(1 << 9)
#define MFT_LSL2B			(1 << 10)
#define MFT_LSL2B_FALL_LATCHED		(1 << 11)

//-------------------------------------
// MFT_SPI_MASTER_CONFIG register bit definitions
//-------------------------------------
#define MFT_SPI_LSCK_DIV_FACTOR_MASK	0xFFF
#define MFT_SPI_MSB_FIRST		(1 << 12)

//-------------------------------------
// MFT_SPI_MASTER_COMMAND register bit definitions
//-------------------------------------
#define MFT_SPI_WR_DATA_MASK		0xFF
#define MFT_SPI_READ			(1 << 8)
#define MFT_SPI_WRITE			0

//-------------------------------------
// MFT_SPI_MASTER_STATUS register bit definitions
//-------------------------------------
#define MFT_SPI_RD_DATA_MASK		0xFF
#define MFT_SPI_BUSY			((uint32_t)(1 << 31))



// ======================================================================================

//-------------------------------------
// IM_ARB Register Space
//-------------------------------------
#define FI_FPGA_IM_ARB_REG(x) (FI_FPGA_BASE_ADDR + FI_FPGA_IM_ARB_OFFSET + (x))

#define IM_ARB_CONFIG_STATUS		FI_FPGA_IM_ARB_REG(0x0100)
#define IM_ARB_PIO_ADDRESS		FI_FPGA_IM_ARB_REG(0x0104)
#define IM_ARB_PIO_WR_DATA		FI_FPGA_IM_ARB_REG(0x0108)
#define IM_ARB_PIO_RD_DATA		FI_FPGA_IM_ARB_REG(0x010C)
#define IM_ARB_PIO_ADDRESS_MSB		FI_FPGA_IM_ARB_REG(0x0110)

#define IM_ARB_BP_THRESHOLDS		FI_FPGA_IM_ARB_REG(0x0120)

#define IM_ARB_PORT_B00_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0140)
#define IM_ARB_PORT_B01_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0144)
#define IM_ARB_PORT_B02_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0148)
#define IM_ARB_PORT_B03_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x014C)
#define IM_ARB_PORT_B04_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0150)
#define IM_ARB_PORT_B05_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0154)
#define IM_ARB_PORT_B06_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0158)
#define IM_ARB_PORT_B07_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x015C)
#define IM_ARB_PORT_B08_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0160)
#define IM_ARB_PORT_B09_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0164)
#define IM_ARB_PORT_B10_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0168)
#define IM_ARB_PORT_B11_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x016C)
#define IM_ARB_PORT_B12_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0170)
#define IM_ARB_PORT_B13_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0174)
#define IM_ARB_PORT_B14_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x0178)
#define IM_ARB_PORT_B15_THROUGHPUT_COUNT	FI_FPGA_MAIN_REG(0x017C)

#define TEMP_SENSOR_CONTROL_STATUS		FI_FPGA_MAIN_REG(0x0200)

//-------------------------------------
// CONFIG_STATUS register bit definitions
//-------------------------------------
#define CONFIG_STATUS_PIO_BUSY		(1 << 0)
#define CONFIG_STATUS_DDR2_INIT_DONE	(1 << 1)
#define CONFIG_STATUS_SPLIT_MEM_MODE	(1 << 4)

//-------------------------------------
// TEMP_SENSOR_CONTROL_STATUS register bit definitions
//-------------------------------------
#define TEMP_SENSOR_TEMPOUT_MASK	0x1FF
#define TEMP_SENSOR_TEMPOUT(x)		(((693.0 * (double)((x) & TEMP_SENSOR_TEMPOUT_MASK)) / 1024.0) - 265.0)
#define TEMP_SENSOR_RESET		(1U << 30)
#define TEMP_SENSOR_CONVERT		(1U << 31)
#define TEMP_SENSOR_BUSY		(1U << 31)

// ======================================================================================

//-------------------------------------
// ICM Register Space
//-------------------------------------
#define FI_FPGA_ICM_REG(x) (FI_FPGA_BASE_ADDR + FI_FPGA_ICM_OFFSET + (x))

#define ICM_CONTROL			FI_FPGA_ICM_REG(0x0000)
#define	ICM_STATUS			FI_FPGA_ICM_REG(0x0004)
#define	ICM_WR_DATA_FIFO_STATUS		FI_FPGA_ICM_REG(0x0008)
// 0x000C unused
#define	ICM_MODULE_RESET		FI_FPGA_ICM_REG(0x0010)
#define	ICM_MODULE_REVISION		FI_FPGA_ICM_REG(0x0014)
// 0x0018-0x001C unused
#define ICM_CAPTURE_START_ADDR		FI_FPGA_ICM_REG(0x0020)
#define ICM_CAPTURE_FRAME_BUF_SIZE	FI_FPGA_ICM_REG(0x0024)
#define ICM_CAPTURE_FRAME_BUF_CNT	FI_FPGA_ICM_REG(0x0028)
#define ICM_POST_TRIG_CNT		FI_FPGA_ICM_REG(0x002C)
#define ICM_ULV_BUF_START_ADDR		FI_FPGA_ICM_REG(0x0030)
#define ICM_BROC_BURST_FRM_CNT		FI_FPGA_ICM_REG(0x0034)
#define ICM_BROC_BURST_START_DELAY_CNT	FI_FPGA_ICM_REG(0x0038)
#define ICM_ROC_MAX_CAPTURE_FRM_CNT	FI_FPGA_ICM_REG(0x003c) // ICM_ROC_BROC_MAX_CAPTURE_FRM_CNT
#define ICM_DMA_BURST_SIZE		FI_FPGA_ICM_REG(0x0040)
// 0x0044-0x007C unused

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// LuximaHS-specific registers. These are pretty much common between the Lux13 and Lux19.
#define ICM_LUX13HS_CONTROL                    	FI_FPGA_ICM_REG(0x0080)
#define ICM_LUX13HS_STATUS                      FI_FPGA_ICM_REG(0x0084)

#define ICM_LUX13HS_SERIAL_CONFIG               FI_FPGA_ICM_REG(0x0088)

#define ICM_LUX13HS_ROI_H_CONFIG                FI_FPGA_ICM_REG(0x008C)
#define ICM_LUX13HS_ROI_V_CONFIG                FI_FPGA_ICM_REG(0x0090)
#define ICM_LUX13HS_1PPS_FRAME_CNT              FI_FPGA_ICM_REG(0x0094)
#define ICM_LUX13HS_1PPS_DELAY_CNT              FI_FPGA_ICM_REG(0x0098)
// 0x009C-0x00BC unused
#define ICM_LUX13HS_LVDS_RX_DPA_RESET_BL        FI_FPGA_ICM_REG(0x00C0)
#define ICM_LUX13HS_LVDS_RX_FIFO_RESET_BL       FI_FPGA_ICM_REG(0x00C4)
#define ICM_LUX13HS_LVDS_RX_BITSLIP_RESET_BL    FI_FPGA_ICM_REG(0x00C8)
#define ICM_LUX13HS_LVDS_RX_BITSLIP_BL          FI_FPGA_ICM_REG(0x00CC)

#define ICM_LUX13HS_LVDS_RX_DPA_RESET_BR        FI_FPGA_ICM_REG(0x00D0)
#define ICM_LUX13HS_LVDS_RX_FIFO_RESET_BR       FI_FPGA_ICM_REG(0x00D4)
#define ICM_LUX13HS_LVDS_RX_BITSLIP_RESET_BR    FI_FPGA_ICM_REG(0x00D8)
#define ICM_LUX13HS_LVDS_RX_BITSLIP_BR          FI_FPGA_ICM_REG(0x00DC)

#define ICM_LUX13HS_LVDS_RX_DPA_RESET_TL        FI_FPGA_ICM_REG(0x00E0)
#define ICM_LUX13HS_LVDS_RX_FIFO_RESET_TL       FI_FPGA_ICM_REG(0x00E4)
#define ICM_LUX13HS_LVDS_RX_BITSLIP_RESET_TL    FI_FPGA_ICM_REG(0x00E8)
#define ICM_LUX13HS_LVDS_RX_BITSLIP_TL          FI_FPGA_ICM_REG(0x00EC)

#define ICM_LUX13HS_LVDS_RX_DPA_RESET_TR        FI_FPGA_ICM_REG(0x00F0)
#define ICM_LUX13HS_LVDS_RX_FIFO_RESET_TR       FI_FPGA_ICM_REG(0x00F4)
#define ICM_LUX13HS_LVDS_RX_BITSLIP_RESET_TR    FI_FPGA_ICM_REG(0x00F8)
#define ICM_LUX13HS_LVDS_RX_BITSLIP_TR          FI_FPGA_ICM_REG(0x00FC)

#define ICM_LUX13HS_LVDS_RX_DPA_LOCKED_BL       FI_FPGA_ICM_REG(0x0100)
#define ICM_LUX13HS_LVDS_RX_DPA_LOCKED_BR       FI_FPGA_ICM_REG(0x0104)
#define ICM_LUX13HS_LVDS_RX_DPA_LOCKED_TL       FI_FPGA_ICM_REG(0x0108)
#define ICM_LUX13HS_LVDS_RX_DPA_LOCKED_TR       FI_FPGA_ICM_REG(0x010C)

#define ICM_LUX13HS_LVDS_RX_PCLK_CH_OUT         FI_FPGA_ICM_REG(0x0110)
// 0x0114-0x013C unused
#define ICM_SENSOR_FRAME_PERIOD                 FI_FPGA_ICM_REG(0x0140)

// Registers 0x144-0x158 are countdown timers from ICM_SENSOR_FRAME_PERIOD to 1.
// The value of '0' means to disregard.
#define ICM_LUX13HS_PD_N_HIGH_COUNT             FI_FPGA_ICM_REG(0x0144)
#define ICM_LUX13HS_PD_N_LOW_COUNT              FI_FPGA_ICM_REG(0x0148)
#define ICM_LUX13HS_TX_N_HIGH_COUNT             FI_FPGA_ICM_REG(0x014C)
#define ICM_LUX13HS_TX_N_LOW_COUNT              FI_FPGA_ICM_REG(0x0150)
#define ICM_LUX13HS_TX2_N_HIGH_COUNT            FI_FPGA_ICM_REG(0x0154)
#define ICM_LUX13HS_TX2_N_LOW_COUNT             FI_FPGA_ICM_REG(0x0158)

#define ICM_LUX13HS_READOUT_TIMING_CONFIG_A     FI_FPGA_ICM_REG(0x015C)
#define ICM_LUX13HS_READOUT_TIMING_CONFIG_B     FI_FPGA_ICM_REG(0x0160)
#define ICM_LUX13HS_READOUT_TIMING_CONFIG_C     FI_FPGA_ICM_REG(0x0164)
#define ICM_LUX13HS_READOUT_TIMING_CONFIG_D     FI_FPGA_ICM_REG(0x0168)
#define ICM_LUX13HS_READOUT_DELAY	        FI_FPGA_ICM_REG(0x016C)
#define ICM_LUX13HS_CLPOUT_CONFIG	        FI_FPGA_ICM_REG(0x0170)
#define ICM_LUX13HS_PD_N_OFFSET		        FI_FPGA_ICM_REG(0x0174)
// LuximaHS-specific registers.
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Lince specific register.
#define ICM_LINCE_CONTROL			FI_FPGA_ICM_REG(0x0080)
#define ICM_LINCE_STATUS			FI_FPGA_ICM_REG(0x0084)

#define ICM_FRAME_TIMING_CONTROL		FI_FPGA_ICM_REG(0x0100)
#define ICM_FRAME_PERIOD			FI_FPGA_ICM_REG(0x0104)
#define ICM_INTEGRATION_TIME			FI_FPGA_ICM_REG(0x0108)
// Lince specific register.
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// 0x0178-0x017C unused
#define ICM_SW_METADATA_TAG			FI_FPGA_ICM_REG(0x0180)
#define ICM_SW_METADATA_MARKER			FI_FPGA_ICM_REG(0x0184)
#define ICM_LAST_FRAME_WRITTEN_PTR 		FI_FPGA_ICM_REG(0x0188)
#define ICM_CAPTURE_FRAME_COUNT_LATCH		FI_FPGA_ICM_REG(0x018C)
#define ICM_CAPTURE_FRAME_COUNT_LIVE		FI_FPGA_ICM_REG(0x0190)
// 0x0194-0x01BC unused
#define ICM_EVENT_FIFO        			FI_FPGA_ICM_REG(0x01C0)
#define ICM_EVENT_FIFO_LEVEL  			FI_FPGA_ICM_REG(0x01C4)
// 0x01C8-0x01DC unused

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// LuximaHS-specific registers.
// Lux13HS test pattern registers
#define ICM_LUX13HS_TP_CONTROL			FI_FPGA_ICM_REG(0x01E0)
#define ICM_LUX13HS_TP_STATUS			FI_FPGA_ICM_REG(0x01E4)
#define ICM_LUX13HS_TP_INIT_FIFO_WRITE		FI_FPGA_ICM_REG(0x01E8)
// 0x01EC unused
#define ICM_LUX13HS_LVDS_RX_STARTUP_READOUT_DELAY		FI_FPGA_ICM_REG(0x01F0)
// 0x01F4-0x01FC unused
#define ICM_LUX13HS_LVDS_INV_BL			FI_FPGA_ICM_REG(0x200)
#define ICM_LUX13HS_LVDS_INV_BR			FI_FPGA_ICM_REG(0x204)
#define ICM_LUX13HS_LVDS_INV_TL			FI_FPGA_ICM_REG(0x208)
#define ICM_LUX13HS_LVDS_INV_TR			FI_FPGA_ICM_REG(0x20C)

#define ICM_LUX13HS_LVDS_MUTE_BL		FI_FPGA_ICM_REG(0x210)
#define ICM_LUX13HS_LVDS_MUTE_BR		FI_FPGA_ICM_REG(0x214)
#define ICM_LUX13HS_LVDS_MUTE_TL		FI_FPGA_ICM_REG(0x218)
#define ICM_LUX13HS_LVDS_MUTE_TR		FI_FPGA_ICM_REG(0x21C)
// 0x220-0x2bc unused
// Lux19HS video trigger registers
#define ICM_LUX19HS_VTRIG_CONTROL			FI_FPGA_ICM_REG(0x02C0)
#define ICM_LUX19HS_VTRIG_STATUS			FI_FPGA_ICM_REG(0x02C4)
#define ICM_LUX19HS_VTRIG_ROI_H_CONFIG_A		FI_FPGA_ICM_REG(0x02C8)
#define ICM_LUX19HS_VTRIG_ROI_H_CONFIG_B		FI_FPGA_ICM_REG(0x02CC)
#define ICM_LUX19HS_VTRIG_ROI_V				FI_FPGA_ICM_REG(0x02D0)
#define ICM_LUX19HS_VTRIG_THRESHOLDS			FI_FPGA_ICM_REG(0x02D4)
#define ICM_LUX19HS_VTRIG_ROI_BUF_UPDATE_INTERVAL	FI_FPGA_ICM_REG(0x02D8)
// LuximaHS-specific registers.
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Lince-specific registers.
#define ICM_1PPS_FRAME_CNT			FI_FPGA_ICM_REG(0x01D0)
#define ICM_1PPS_DELAY_CNT			FI_FPGA_ICM_REG(0x01D4)
// 0x01D8-0x0204 unused
#define ICM_LINCE_POWER_ENABLE			FI_FPGA_ICM_REG(0x0208)
#define ICM_LINCE_ROI_H_CONFIG			FI_FPGA_ICM_REG(0x020C)
#define ICM_LINCE_ROI_V_CONFIG			FI_FPGA_ICM_REG(0x0210)
// 0x214-0x21C unused
#define ICM_LINCE_LVDS_RX_DPA_RESET		FI_FPGA_ICM_REG(0x0220)
#define ICM_LINCE_LVDS_RX_DPA_LOCKED		FI_FPGA_ICM_REG(0x0224)
#define ICM_LINCE_LVDS_RX_FIFO_RESET		FI_FPGA_ICM_REG(0x0228)
#define ICM_LINCE_LVDS_RX_BITSLIP_RESET		FI_FPGA_ICM_REG(0x022C)
#define ICM_LINCE_LVDS_RX_BITSLIP		FI_FPGA_ICM_REG(0x0230)
#define ICM_LINCE_LVDS_CH_INV			FI_FPGA_ICM_REG(0x0234)
// 0x238-0x23C unused
#define ICM_LINCE_LVDS_RX_CH00_03_DATA		FI_FPGA_ICM_REG(0x0240)
#define ICM_LINCE_LVDS_RX_CH04_07_DATA		FI_FPGA_ICM_REG(0x0244)
#define ICM_LINCE_LVDS_RX_CH08_11_DATA		FI_FPGA_ICM_REG(0x0248)
#define ICM_LINCE_LVDS_RX_CH12_15_DATA		FI_FPGA_ICM_REG(0x024C)
#define ICM_LINCE_LVDS_RX_CH16_19_DATA		FI_FPGA_ICM_REG(0x0250)
#define ICM_LINCE_LVDS_RX_CH20_23_DATA		FI_FPGA_ICM_REG(0x0254)
#define ICM_LINCE_LVDS_RX_SYNC_DATA		FI_FPGA_ICM_REG(0x0258)
// 0x25C-0x27C unused
#define ICM_LINCE_SPI_CONTROL			FI_FPGA_ICM_REG(0x0280)
#define ICM_LINCE_SPI_COMMAND			FI_FPGA_ICM_REG(0x0284)
#define ICM_LINCE_SPI_REG_ADDRESS		FI_FPGA_ICM_REG(0x0288)
#define ICM_LINCE_SPI_DATA_BYTE_COUNT		FI_FPGA_ICM_REG(0x028C)
#define ICM_LINCE_SPI_WR_DATA_FIFO		FI_FPGA_ICM_REG(0x0290)
#define ICM_LINCE_SPI_WR_DATA_FIFO_LEVEL	FI_FPGA_ICM_REG(0x0294)
#define ICM_LINCE_SPI_RD_DATA_FIFO		FI_FPGA_ICM_REG(0x0298)
#define ICM_LINCE_SPI_RD_DATA_FIFO_LEVEL	FI_FPGA_ICM_REG(0x029C)
// 0x2a0-0x2bc unused
#define ICM_LINCE_VTRIG_CONTROL			FI_FPGA_ICM_REG(0x02C0)
#define ICM_LINCE_VTRIG_STATUS			FI_FPGA_ICM_REG(0x02C4)
#define ICM_LINCE_VTRIG_ROI_H_CONFIG_A		FI_FPGA_ICM_REG(0x02C8)
#define ICM_LINCE_VTRIG_ROI_H_CONFIG_B		FI_FPGA_ICM_REG(0x02CC)
#define ICM_LINCE_VTRIG_ROI_V			FI_FPGA_ICM_REG(0x02D0)
#define ICM_LINCE_VTRIG_THRESHOLDS		FI_FPGA_ICM_REG(0x02D4)
#define ICM_LINCE_VTRIG_ROI_BUF_UPDATE_INTERVAL	FI_FPGA_ICM_REG(0x02D8)
// Lince-specific registers.
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
//xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
//xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

// Allow ICM addresses up to 0x0400.
#define ICM_LAST_REG				FI_FPGA_ICM_REG(0x0400)

//-------------------------------------
// ICM_CONTROL register bit definitions
//-------------------------------------
#define ICMC_CAPTURE_MODE_MASK		(3 << 4)
#define	    ICMC_CAPTURE_MODE_BROC		(2 << 4)
#define	    ICMC_CAPTURE_MODE_ROC		(1 << 4)
#define	    ICMC_CAPTURE_MODE_BASIC		(0 << 4)
#define ICMC_PIPELINE_ENABLE   		(1 << 24)
#define ICMC_ARM	   		(1 << 25)
#define ICMC_UNARM	   		(1 << 26)
#define ICMC_CAPTURE_SINGLE   		(1 << 27)
#define ICMC_TRIGGER	   		(1 << 28)
#define ICMC_LATCHED_TRIGGER_CLR	(1 << 29)
#define ICMC_LR_ENABLE			(1 << 30)

//-------------------------------------
// ICM_STATUS register bit definitions
//-------------------------------------
#define ICMS_CAPTURE_SINGLE_BUSY	(1 << 0)
#define ICMS_PIPELINE_IDLE		(1 << 8)
#define ICMS_PIPELINE_READY		(1 << 9)
#define ICMS_CAPTURE_IN_PROGRESS	(1 << 10)
#define ICMS_ARMED			(1 << 11)
#define ICMS_POST_TRIG_CAPTURE		(1 << 12)
#define ICMS_SYNC_1PPS_COLLISION	(1 << 13)
#define ICMS_ROC_BROC_PAUSED		(1 << 14)
#define ICMS_ROC_BROC_RECORD		(1 << 15)
#define ICMS_DMA_BUSY			(1 << 21)

//-------------------------------------
// ICM_WR_DATA_FIFO_STATUS register bit definitions
//-------------------------------------
#define ICMWDFS_FIFO_LEVEL_MAX_MASK	(0xFF << 0)
#define ICMWDFS_FIFO_OVERFLOW		(1 << 16)
#define ICMWDFS_RESET			0x10001

//-------------------------------------
// ICM_FRAME_TIMING_CONTROL register bit definitions
//-------------------------------------
#define ICMFTC_SENSOR_READOUT_ENA		(1 << 0)
#define ICMFTC_SYNC_MODE_MASK			(3 << 4)
#define ICMFTC_SYNC_MODE_MASTER				(0 << 4)
#define ICMFTC_SYNC_MODE_1PPF_SLAVE			(1 << 4)
#define ICMFTC_SYNC_MODE_1PPS_SLAVE			(2 << 4)
#define ICMFTC_SYNC_IN_1PPF_SKIP_CNT_SHIFT	8
#define ICMFTC_SYNC_IN_1PPF_SKIP_CNT_MASK		(7 << ICMFTC_SYNC_IN_1PPF_SKIP_CNT_SHIFT)
#define ICMFTC_SYNC_IN_1PPF_SKIP_CNT(x)			(((x) & 7) << ICMFTC_SYNC_IN_1PPF_SKIP_CNT_SHIFT)
#define ICMFTC_SW_SYNC_PULSE			(1U << 31)

//-------------------------------------
// ICM_LUX13HS_CONTROL register bit definitions
//-------------------------------------
#define ICMX13HSC_SENSOR_RESET			(1 << 0)
#define ICMX13HSC_SENSOR_PLL_ENA		(1 << 1)
#define ICMX13HSC_SENSOR_READOUT_ENA		(1 << 2)
#define ICMX13HSC_ALTLVDS_RX_RESET_BOT		(1 << 4)
#define ICMX13HSC_ALTLVDS_RX_PLL_ARESET_BOT	(1 << 5)
#define ICMX13HSC_ALTLVDS_RX_PAR_OUT_ENA_BOT	(1 << 6)
#define ICMX13HSC_ALTLVDS_RX_RESET_TOP		(1 << 8)
#define ICMX13HSC_ALTLVDS_RX_PLL_ARESET_TOP	(1 << 9)
#define ICMX13HSC_ALTLVDS_RX_PAR_OUT_ENA_TOP	(1 << 10)
#define ICMX13HSC_SYNC_MODE_MASTER		(0 << 12)
#define ICMX13HSC_SYNC_MODE_1PPF_SLAVE		(1 << 12)
#define ICMX13HSC_SYNC_MODE_1PPS_SLAVE		(2 << 12)
#define ICMX13HSC_SYNC_MODE_MASK		(3 << 12)
#define ICMX13HSC_BIT_DEPTH_8 			(0 << 16)
#define ICMX13HSC_BIT_DEPTH_10			(1 << 16)
#define ICMX13HSC_BIT_SEL_MASK			(3 << 18)
#define ICMX13HSC_BIT_SEL_7_0			(0 << 18)
#define ICMX13HSC_BIT_SEL_8_1			(1 << 18)
#define ICMX13HSC_BIT_SEL_9_2			(2 << 18)
#define ICMX13HSC_1PPF_SKIP_COUNT_MASK		(7 << 20)
#define ICMX13HSC_1PPF_SKIP_COUNT(x)		(((x) << 20) & ICMX13HSC_1PPF_SKIP_COUNT_MASK)
#define ICMX13HSC_ALTLVDS_RX_RESET		(ICMX13HSC_ALTLVDS_RX_RESET_TOP | ICMX13HSC_ALTLVDS_RX_RESET_BOT)
#define ICMX13HSC_ALTLVDS_RX_PAR_OUT_ENA	(ICMX13HSC_ALTLVDS_RX_PAR_OUT_ENA_TOP | ICMX13HSC_ALTLVDS_RX_PAR_OUT_ENA_BOT)

//-------------------------------------
// ICM_LUX13HS_STATUS register bit definitions
//-------------------------------------
#define ICMX13HSS_ALTLVDS_RX_PLL_LOCKED_BL      (1 << 0)
#define ICMX13HSS_ALTLVDS_RX_PLL_LOCKED_BR      (1 << 1)
#define ICMX13HSS_ALTLVDS_RX_PLL_LOCKED_TL      (1 << 2)
#define ICMX13HSS_ALTLVDS_RX_PLL_LOCKED_TR      (1 << 3)
#define ICMX13HSS_ALTLVDS_RX_PLL_LOCKED		0xF
#define ICMX13HSS_BLOCK_FIFO_LEVEL_SHIFT        16
#define ICMX13HSS_BLOCK_FIFO_LEVEL_MASK         (0x7F << ICMX13HSS_BLOCK_FIFO_LEVEL_SHIFT)
#define ICMX13HSS_CONFIG_BUSY                   ((uint32_t)(1 << 31))

//-------------------------------------
// ICM_LUX13HS_ROI_H_CONFIG register bit definitions
//-------------------------------------
#define ICMX13HSROIH_GET_FIRST_ACTIVE_BLOCK(x)              ((x) & 0x3F)
#define ICMX13HSROIH_GET_FIRST_ACTIVE_BLOCK_OFFSET(x)       (((x) >> 8) & 0x3F)
#define ICMX13HSROIH_GET_LAST_ACTIVE_BLOCK_OFFSET(x)        (((x) >> 16) & 0x3F)
#define ICMX13HSROIH_GET_LAST_ACTIVE_BLOCK_PIXEL_COUNT(x)   (((x) >> 24) & 0x3F)
#define ICMX13HSROIH_SET_FIRST_ACTIVE_BLOCK(r, x)              (((r) & ~0x3F) | ((x) & 0x3F))
#define ICMX13HSROIH_SET_FIRST_ACTIVE_BLOCK_OFFSET(r, x)       (((r) & ~(0x3F << 8)) | (((x) & 0x3F) << 8))
#define ICMX13HSROIH_SET_LAST_ACTIVE_BLOCK_OFFSET(r, x)        (((r) & ~(0x3F << 16)) | (((x) & 0x3F) << 16))
#define ICMX13HSROIH_SET_LAST_ACTIVE_BLOCK_PIXEL_COUNT(r, x)   (((r) & ~(0x3F << 24)) | (((x) & 0x3F) << 24))

//-------------------------------------
// ICM_LUX13HS_READOUT_TIMING_CONFIG_B register bit definitions
//-------------------------------------
#define ICMX13HSCFGB_GET_FIRST_QROW(x)              ((x) & 0xFF)
#define ICMX13HSCFGB_GET_QROW_COUNT(x)              (((x) >> 8) & 0xFF)
#define ICMX13HSCFGB_GET_QROW_TIME(x)               (((x) >> 16) & 0xFF)
#define ICMX13HSCFGB_SET_FIRST_QROW(r, x)           (((r) & ~0xFF) | ((x) & 0xFF))
#define ICMX13HSCFGB_SET_QROW_COUNT(r, x)           (((r) & ~(0xFF << 8)) | (((x) & 0xFF) << 8))
#define ICMX13HSCFGB_SET_QROW_TIME(r, x)            (((r) & ~(0xFF << 16)) | (((x) & 0xFF) << 16))

#define ICMX19HSCFGB_SET_FIRST_QROW(r, x)           (((r) & ~0x1FF) | ((x) & 0x1FF))
#define ICMX19HSCFGB_SET_QROW_COUNT(r, x)           (((r) & ~(0x1FF << 12)) | (((x) & 0x1FF) << 12))
#define ICMX19HSCFGB_SET_QROW_TIME(r, x)            (((r) & ~(0xFF << 24)) | (((x) & 0xFF) << 24))

//-------------------------------------
// ICM_LUX13HS_READOUT_TIMING_CONFIG_C register bit definitions
//-------------------------------------
#define ICMX13HSCFGC_GET_ST_ROW_BOT_N_COUNT(x)      ((x) & 0xFF)
#define ICMX13HSCFGC_GET_ST_READ_BOT_N_COUNT(x)     (((x) >> 8) & 0xFF)
#define ICMX13HSCFGC_GET_ROW_ADDR_INC_COUNT(x)      (((x) >> 16) & 0xFF)
#define ICMX13HSCFGC_SET_ST_ROW_BOT_N_COUNT(r, x)   (((r) & ~0xFF) | ((x) & 0xFF))
#define ICMX13HSCFGC_SET_ST_READ_BOT_N_COUNT(r, x)  (((r) & ~(0xFF << 8)) | (((x) & 0xFF) << 8))
#define ICMX13HSCFGC_SET_ROW_ADDR_INC_COUNT(r, x)   (((r) & ~(0xFF << 16)) | (((x) & 0xFF) << 16))

//-------------------------------------
// ICM_LUX13HS_READOUT_TIMING_CONFIG_D register bit definitions
//-------------------------------------
#define ICMX13HSCFGD_GET_ST_ROW_TOP_N_COUNT(x)      ((x) & 0xFF)
#define ICMX13HSCFGD_GET_ST_READ_TOP_N_COUNT(x)     (((x) >> 8) & 0xFF)
#define ICMX13HSCFGD_SET_ST_ROW_TOP_N_COUNT(r, x)   (((r) & ~0xFF) | ((x) & 0xFF))
#define ICMX13HSCFGD_SET_ST_READ_TOP_N_COUNT(r, x)  (((r) & ~(0xFF << 8)) | (((x) & 0xFF) << 8))


//-------------------------------------
// ICM_LUX13HS_TP_CONTROL register bit definitions
//-------------------------------------
#define ICMX13HSTPC_ENABLE		(1 << 0)
#define ICMX13HSTPC_RESET		(1 << 1)
#define ICMX13HSTPC_START		(1 << 2)
#define ICMX13HSTPC_ROW_SEL_MASK	0x70
#define ICMX13HSTPC_ROW_SEL_SHIFT	4
#define ICMX13HSTPC_SET_ROW_SEL(r, x)	(((r) & ~ICMX13HSTPC_ROW_SEL_MASK) | (((x) << 4) & ICMX13HSTPC_ROW_SEL_MASK))

//-------------------------------------
// ICM_LUX13HS_TP_STATUS register bit definitions
//-------------------------------------
#define ICMX13HSTPS_BUSY		((uint32_t)(1 << 31))
#define ICMX13HSTPS_GET_FIFO_LEVEL(r)	((r) & 0x7FF)


//-------------------------------------
// ICM_LUX13HS_CLPOUT_CONFIG register bit definitions
//-------------------------------------
#define ICMX13HS_CLPOUT_PULSE2_OFFSET_MASK	0x1F

// The pulse width is "+1", so specifying a 1 equates to a pulse width of 2 clocks.
#define ICMX13HS_CLPOUT_PULSE_WIDTH_MASK	(3 << 8)
#define ICMX13HS_CLPOUT_PULSE_WIDTH(x)		((x) << 8)
#define ICMX13HS_CLPOUT_DISABLE			0
#define ICMX13HS_CLPOUT_ENABLE			(1 << 12)

//-------------------------------------
// ICM_EVENT_FIFO register bit definitions
//-------------------------------------
#define ICMEF_TRIGGER_FRAME_MASK	0x3FFFFFFF
#define ICMEF_TRIGGER_FLAG		(1 << 31)

// Word 2 of the event FIFO (only if the trigger flag is '1').
#define ICMEF_SBS_TIME_MASK		0xFFFFFFFF

// Word 3 of the event FIFO (only if the trigger flag is '1').
#define ICMEF_SBUS_TIME_MASK		0xFFFFF
#define ICMEF_EXT_MARK_SHIFT		24
#define ICMEF_EXT_MARK_MASK		(0x3F << ICMEF_EXT_MARK_SHIFT)
#define ICMEF_ARM_OUT_DIG_IN2		(1 << 24)
#define ICMEF_ARM_IN_DIG_IN3		(1 << 25)
#define ICMEF_SYNC_OUT_DIG_IN6		(1 << 26)
#define ICMEF_SYNC_IN_DIG_IN5		(1 << 27)
#define ICMEF_TRIG_OUT_DIG_IN1		(1 << 28)
#define ICMEF_TRIG_IN_DIG_IN4		(1 << 29)
#define ICMEF_IRIG_LOCK			(1 << 31)

//-------------------------------------------
// ICM_LINCE_CONTROL register bit definitions
//-------------------------------------------
#define ILC_SENSOR_RESET		( 1 << 0 )
#define ILC_LVDS_RX_RESET		( 1 << 1 )
#define ILC_LVDS_RX_PLL_ARESET		( 1 << 2 )
#define ILC_LVDS_RX_PAR_OUT_VALID	( 1 << 3 )
#define ILC_LVDS_RX_IDELAYCTRL_RESET	( 1 << 4 )
// Bits 4:5 reserved
#define ILC_BIT_DEPTH_MASK		( 3 << 6 )  // bits 6:7
#define ILC_BIT_DEPTH_8         ( 0 << 6 )
#define ILC_BIT_DEPTH_10        ( 1 << 6 )
#define ILC_BIT_DEPTH_12        ( 2 << 6 )
#define ILC_BIT_SEL_MASK		( 7 << 8 )  // bits 8:10
#define ILC_BIT_SEL_7_0		( 0 << 8 )
#define ILC_BIT_SEL_8_1         ( 1 << 8 )
#define ILC_BIT_SEL_9_2         ( 2 << 8 )
#define ILC_BIT_SEL_10_3        ( 3 << 8 )
#define ILC_BIT_SEL_11_4        ( 4 << 8 )
#define ILC_BIT_SEL_9_0		( 0 << 8 )
#define ILC_BIT_SEL_10_1        ( 1 << 8 )
#define ILC_BIT_SEL_11_2        ( 2 << 8 )
#define ILC_TP_ENABLE			( 1 << 16 ) // Set to turn on ICM test pattern.

//-------------------------------------------
// ICM_LINCE_STATUS register bit definitions
//-------------------------------------------
#define ILS_LVDS_RX_PLL_LOCKED	    ( 1 << 0 )
#define ILS_LVDS_RX_IDELAYCTRL_RDY1 ( 1 << 1 )
#define ILS_LVDS_RX_IDELAYCTRL_RDY2 ( 1 << 2 )
#define ILS_PPW_FIFO_LEVEL_MASK	    0x01ff  // bits 16:24
#define ILS_PPW_FIFO_LEVEL_SHIFT    24
#define ILS_SPI_BUSY			( 1 << 31 )

//-----------------------------------------------
// ICM_LINCE_ROI_H_CONFIG register bit definitions
//-----------------------------------------------
#define ILRH_PPWS_PER_ROW_MASK	    0x000000ff
#define ILRH_PPWS_PER_ROW_SHIFT	    0
#define ILRH_FIRST_PPW_OFFSET_MASK  0x00003f00
#define ILRH_FIRST_PPW_OFFSET_SHIFT 8
#define ILRH_LAST_PPW_COUNT_MASK    0x07f00000
#define ILRH_LAST_PPW_COUNT_SHIFT   20

//-----------------------------------------------
// ICM_LINCE_LVDS_RX_CONTROL register bit definitions
//-----------------------------------------------
#define ILRC_IDELAY_RESET	(1 << 0 )   // Reset delay elements
#define ILRC_ISERDES_BITSLIP	(1 << 1 )   // Do one bitslip
#define ILRC_ISERDES_RESET	(1 << 2 )   // Reset bitslip

//-----------------------------------------------
// ICM_LINCE_SPI_CONTROL register bit definitions
//-----------------------------------------------
#define ILSC_ADDRESS_ENA    	(1 << 0)
#define ILSC_DUMMY_ENA	    	(1 << 1)
#define ILSC_WR_DIRECTION   	(0 << 2)
#define ILSC_RD_DIRECTION   	(1 << 2)
#define ILSC_WR_FIFO_RESET  	(1 << 4)
#define ILSC_RD_FIFO_RESET  	(1 << 5)

//-----------------------------------------------
// ICM_LINCE_VTRIG_CONTROL register bit definitions
// ICM_LUX19HS_VTRIG_CONTROL register bit definitions
//-----------------------------------------------
#define ILVT_MODULE_ENABLE	(1 << 0)
#define ILVT_TRIGGER_ENABLE	(1 << 1)

//-----------------------------------------------
// ICM_LINCE_VTRIG_STATUS register bit definitions
// ICM_LUX19HS_VTRIG_STATUS register bit definitions
//-----------------------------------------------
#define ILVT_BUSY		(1 << 0)
#define ILVT_FF_BUSY		(1 << 1)

//-----------------------------------------------
// ICM_LINCE_VTRIG_ROI_H_CONFIG_A register bit definitions
// ICM_LUX19HS_VTRIG_ROI_H_CONFIG_A register bit definitions
//-----------------------------------------------
#define ILVT_FIRST_PPW_START_MASK	((uint32_t)0x000F)
#define ILVT_LAST_PPW_PIXEL_MASK	((uint32_t)0x01F0)
#define ILVT_SET_ROI_H_CONFIG_A(first, last)	((uint32_t)(((first) & ILVT_FIRST_PPW_START_MASK) | (((last) << 4) & ILVT_LAST_PPW_PIXEL_MASK)))
#define IL19VT_FIRST_PPW_START_MASK	((uint32_t)0x001F)
#define IL19VT_LAST_PPW_PIXEL_MASK	((uint32_t)0x3F00)
#define IL19VT_SET_ROI_H_CONFIG_A(first, last)	((uint32_t)(((first) & IL19VT_FIRST_PPW_START_MASK) | (((last) << 8) & IL19VT_LAST_PPW_PIXEL_MASK)))

//-----------------------------------------------
// ICM_LINCE_VTRIG_ROI_H_CONFIG_B register bit definitions
// ICM_LUX19HS_VTRIG_ROI_H_CONFIG_B register bit definitions
//-----------------------------------------------
#define ILVT_FIRST_ACTIVE_PPW_MASK	((uint32_t)0x00FF)
#define ILVT_LAST_ACTIVE_PPW_MASK	((uint32_t)0xFF00)
#define ILVT_SET_ROI_H_CONFIG_B(first, last)	((uint32_t)(((first) & ILVT_FIRST_ACTIVE_PPW_MASK) | (((last) << 8) & ILVT_LAST_ACTIVE_PPW_MASK)))
#define IL19VT_FIRST_ACTIVE_PPW_MASK	((uint32_t)0x003F)
#define IL19VT_LAST_ACTIVE_PPW_MASK	((uint32_t)0x3F00)
#define IL19VT_SET_ROI_H_CONFIG_B(first, last)	((uint32_t)(((first) & IL19VT_FIRST_ACTIVE_PPW_MASK) | (((last) << 8) & IL19VT_LAST_ACTIVE_PPW_MASK)))

//-----------------------------------------------
// ICM_LINCE_VTRIG_ROI_V register bit definitions
// ICM_LUX19HS_VTRIG_ROI_V register bit definitions
//-----------------------------------------------
#define ILVT_LAST_ROW_SHIFT		16
#define ILVT_FIRST_ROW_MASK		((uint32_t)0x00000FFF)
#define ILVT_LAST_ROW_MASK		((uint32_t)0x0FFF0000)
#define ILVT_SET_ROI_V(first, last)	((uint32_t)(((first) & ILVT_FIRST_ROW_MASK) | (((last) << 16) & ILVT_LAST_ROW_MASK)))
#define IL19VT_FIRST_ROW_MASK		((uint32_t)0x000007FF)
#define IL19VT_LAST_ROW_MASK		((uint32_t)0x07FF0000)
#define IL19VT_SET_ROI_V(first, last)	((uint32_t)(((first) & IL19VT_FIRST_ROW_MASK) | (((last) << 16) & IL19VT_LAST_ROW_MASK)))

//-----------------------------------------------
// ICM_LINCE_VTRIG_THRESHOLDS register bit definitions
// ICM_LUX19HS_VTRIG_THRESHOLDS register bit definitions
//-----------------------------------------------
#define ILVT_CHANGE_THRESHOLD_MASK	((uint32_t)0x000000FF)
#define ILVT_COUNT_THRESHOLD_MASK	((uint32_t)0x03FFFF00)
#define ILVT_COUNT_THRESHOLD_SHIFT	8
#define ILVT_SET_THRESHOLD(change, count)	((uint32_t)(((change) & ILVT_CHANGE_THRESHOLD_MASK) | (((count) << 8) & ILVT_COUNT_THRESHOLD_MASK)))

//-----------------------------------------------
// ICM_LINCE_VTRIG_ROI_BUF_UPDATE_INTERVAL register bit definitions
// ICM_LUX19HS_VTRIG_ROI_BUF_UPDATE_INTERVAL register bit definitions
//-----------------------------------------------
#define ILVT_SET_UPDATE_INTERVAL(val)	((uint32_t)((val) & 0xFFFF))

// ======================================================================================

//-------------------------------------
// IPM Register Space
//-------------------------------------
#define FI_FPGA_IPM0_REG(x) (FI_FPGA_BASE_ADDR + FI_FPGA_IPM0_OFFSET + (x))
#define FI_FPGA_IPM1_REG(x) (FI_FPGA_BASE_ADDR + FI_FPGA_IPM1_OFFSET + (x))

#define IPM_CONTROL		        FI_FPGA_IPM0_REG(0x0000)
#define IPM_STATUS		        FI_FPGA_IPM0_REG(0x0004)
#define IPM_SRC_DMA_BURST_SIZE	        FI_FPGA_IPM0_REG(0x0008)
#define IPM_SRC_DMA_BP_THRESHOLD        FI_FPGA_IPM0_REG(0x000C)
#define IPM_CT_DMA_BURST_SIZE           FI_FPGA_IPM0_REG(0x0010)
#define IPM_CT_DMA_BP_THRESHOLD         FI_FPGA_IPM0_REG(0x0014)
#define IPM_DEST_DMA_BURST_SIZE         FI_FPGA_IPM0_REG(0x0018)
#define IPM_DEST_DMA_BP_THRESHOLD       FI_FPGA_IPM0_REG(0x001C)
#define IPM_HISTO_ROI_X_CONFIG          FI_FPGA_IPM0_REG(0x0020)
#define IPM_HISTO_ROI_Y_CONFIG          FI_FPGA_IPM0_REG(0x0024)
#define IPM_HISTO_PEAK_VALUE            FI_FPGA_IPM0_REG(0x0028)
#define IPM_HISTO_SUM_OF_X_RED          FI_FPGA_IPM0_REG(0x002C)
#define IPM_HISTO_SUM_OF_X_GRN          FI_FPGA_IPM0_REG(0x0030)
#define IPM_HISTO_SUM_OF_X_BLU          FI_FPGA_IPM0_REG(0x0034)
#define IPM_HISTO_SUM_OF_X_SQRD_RED_H   FI_FPGA_IPM0_REG(0x0038)
#define IPM_HISTO_SUM_OF_X_SQRD_RED_L   FI_FPGA_IPM0_REG(0x003C)
#define IPM_HISTO_SUM_OF_X_SQRD_GRN_H   FI_FPGA_IPM0_REG(0x0040)
#define IPM_HISTO_SUM_OF_X_SQRD_GRN_L   FI_FPGA_IPM0_REG(0x0044)
#define IPM_HISTO_SUM_OF_X_SQRD_BLU_H   FI_FPGA_IPM0_REG(0x0048)
#define IPM_HISTO_SUM_OF_X_SQRD_BLU_L   FI_FPGA_IPM0_REG(0x004C)
// 0x50-0x7C are reserved.
#define IPM_P0_CONFIG                   FI_FPGA_IPM0_REG(0x0080)
#define IPM_P0_STATUS                   FI_FPGA_IPM0_REG(0x0084)
#define IPM_P0_SRC_DMA_BLOCK_SIZE       FI_FPGA_IPM0_REG(0x0088)
#define IPM_P0_CT_DMA_START_ADDR        FI_FPGA_IPM0_REG(0x008C)
#define IPM_P0_CT_DMA_BLOCK_SIZE        FI_FPGA_IPM0_REG(0x0090)
#define IPM_P0_FPN_CORR_THRESHOLD	FI_FPGA_IPM0_REG(0x0094)
#define IPM_P0_FRAME_DIM                FI_FPGA_IPM0_REG(0x0098)
#define IPM_P0_WB_GAIN_FACTOR_RED	FI_FPGA_IPM0_REG(0x009C)
#define IPM_P0_WB_GAIN_FACTOR_GRN	FI_FPGA_IPM0_REG(0x00A0)
#define IPM_P0_WB_GAIN_FACTOR_BLU	FI_FPGA_IPM0_REG(0x00A4)
#define IPM_P0_CC_MATRIX_COEFF1_RED     FI_FPGA_IPM0_REG(0x00A8)
#define IPM_P0_CC_MATRIX_COEFF2_RED     FI_FPGA_IPM0_REG(0x00AC)
#define IPM_P0_CC_MATRIX_COEFF3_RED     FI_FPGA_IPM0_REG(0x00B0)
#define IPM_P0_CC_MATRIX_COEFF1_GRN     FI_FPGA_IPM0_REG(0x00B4)
#define IPM_P0_CC_MATRIX_COEFF2_GRN     FI_FPGA_IPM0_REG(0x00B8)
#define IPM_P0_CC_MATRIX_COEFF3_GRN     FI_FPGA_IPM0_REG(0x00BC)
#define IPM_P0_CC_MATRIX_COEFF1_BLU     FI_FPGA_IPM0_REG(0x00C0)
#define IPM_P0_CC_MATRIX_COEFF2_BLU     FI_FPGA_IPM0_REG(0x00C4)
#define IPM_P0_CC_MATRIX_COEFF3_BLU     FI_FPGA_IPM0_REG(0x00C8)
#define IPM_P0_RGB_GAIN_RED             FI_FPGA_IPM0_REG(0x00CC)
#define IPM_P0_RGB_GAIN_GRN             FI_FPGA_IPM0_REG(0x00D0)
#define IPM_P0_RGB_GAIN_BLU             FI_FPGA_IPM0_REG(0x00D4)
#define IPM_P0_GT_DMA_START_ADDR        FI_FPGA_IPM0_REG(0x00D8)
#define IPM_P0_GT_DMA_BLOCK_SIZE        FI_FPGA_IPM0_REG(0x00DC)
// 0xD8-0xF7 are reserved.
#define IPM_P0_RAW_PIXEL_OFFSET_GAIN	FI_FPGA_IPM0_REG(0x00F8)
// 0xFC-0xFF are reserved.
#define IPM_P1_CONFIG                   FI_FPGA_IPM0_REG(0x0100)
#define IPM_P1_STATUS                   FI_FPGA_IPM0_REG(0x0104)
#define IPM_P1_SRC_DMA_BLOCK_SIZE       FI_FPGA_IPM0_REG(0x0108)
#define IPM_P1_CT_DMA_START_ADDR        FI_FPGA_IPM0_REG(0x010C)
#define IPM_P1_CT_DMA_BLOCK_SIZE        FI_FPGA_IPM0_REG(0x0110)
#define IPM_P1_FPN_CORR_THRESHOLD       FI_FPGA_IPM0_REG(0x0114)
#define IPM_P1_FRAME_DIM                FI_FPGA_IPM0_REG(0x0118)
#define IPM_P1_WB_GAIN_FACTOR_RED       FI_FPGA_IPM0_REG(0x011C)
#define IPM_P1_WB_GAIN_FACTOR_GRN	FI_FPGA_IPM0_REG(0x0120)
#define IPM_P1_WB_GAIN_FACTOR_BLU       FI_FPGA_IPM0_REG(0x0124)
#define IPM_P1_CC_MATRIX_COEFF1_RED     FI_FPGA_IPM0_REG(0x0128)
#define IPM_P1_CC_MATRIX_COEFF2_RED     FI_FPGA_IPM0_REG(0x012C)
#define IPM_P1_CC_MATRIX_COEFF3_RED     FI_FPGA_IPM0_REG(0x0130)
#define IPM_P1_CC_MATRIX_COEFF1_GRN     FI_FPGA_IPM0_REG(0x0134)
#define IPM_P1_CC_MATRIX_COEFF2_GRN     FI_FPGA_IPM0_REG(0x0138)
#define IPM_P1_CC_MATRIX_COEFF3_GRN     FI_FPGA_IPM0_REG(0x013C)
#define IPM_P1_CC_MATRIX_COEFF1_BLU     FI_FPGA_IPM0_REG(0x0140)
#define IPM_P1_CC_MATRIX_COEFF2_BLU     FI_FPGA_IPM0_REG(0x0144)
#define IPM_P1_CC_MATRIX_COEFF3_BLU     FI_FPGA_IPM0_REG(0x0148)
#define IPM_P1_RGB_GAIN_RED             FI_FPGA_IPM0_REG(0x014C)
#define IPM_P1_RGB_GAIN_GRN             FI_FPGA_IPM0_REG(0x0150)
#define IPM_P1_RGB_GAIN_BLU             FI_FPGA_IPM0_REG(0x0154)
#define IPM_P1_GT_DMA_START_ADDR        FI_FPGA_IPM0_REG(0x0158)
#define IPM_P1_GT_DMA_BLOCK_SIZE        FI_FPGA_IPM0_REG(0x015C)
// 0x158-0x177 are reserved.
#define IPM_P1_RAW_PIXEL_OFFSET_GAIN	FI_FPGA_IPM0_REG(0x0178)
// 0x17C-0x17C are reserved.
#define IPM_P2_CONFIG                   FI_FPGA_IPM0_REG(0x0180)
#define IPM_P2_STATUS                   FI_FPGA_IPM0_REG(0x0184)
#define IPM_P2_SRC_DMA_BLOCK_SIZE       FI_FPGA_IPM0_REG(0x0188)
#define IPM_P2_CT_DMA_START_ADDR        FI_FPGA_IPM0_REG(0x018C)
#define IPM_P2_CT_DMA_BLOCK_SIZE        FI_FPGA_IPM0_REG(0x0190)
#define IPM_P2_FPN_CORR_THRESHOLD       FI_FPGA_IPM0_REG(0x0194)
#define IPM_P2_FRAME_DIM                FI_FPGA_IPM0_REG(0x0198)
#define IPM_P2_WB_GAIN_FACTOR_RED       FI_FPGA_IPM0_REG(0x019C)
#define IPM_P2_WB_GAIN_FACTOR_GRN       FI_FPGA_IPM0_REG(0x01A0)
#define IPM_P2_WB_GAIN_FACTOR_BLU       FI_FPGA_IPM0_REG(0x01A4)
#define IPM_P2_CC_MATRIX_COEFF1_RED     FI_FPGA_IPM0_REG(0x01A8)
#define IPM_P2_CC_MATRIX_COEFF2_RED     FI_FPGA_IPM0_REG(0x01AC)
#define IPM_P2_CC_MATRIX_COEFF3_RED     FI_FPGA_IPM0_REG(0x01B0)
#define IPM_P2_CC_MATRIX_COEFF1_GRN     FI_FPGA_IPM0_REG(0x01B4)
#define IPM_P2_CC_MATRIX_COEFF2_GRN     FI_FPGA_IPM0_REG(0x01B8)
#define IPM_P2_CC_MATRIX_COEFF3_GRN     FI_FPGA_IPM0_REG(0x01BC)
#define IPM_P2_CC_MATRIX_COEFF1_BLU     FI_FPGA_IPM0_REG(0x01C0)
#define IPM_P2_CC_MATRIX_COEFF2_BLU     FI_FPGA_IPM0_REG(0x01C4)
#define IPM_P2_CC_MATRIX_COEFF3_BLU     FI_FPGA_IPM0_REG(0x01C8)
#define IPM_P2_RGB_GAIN_RED             FI_FPGA_IPM0_REG(0x01CC)
#define IPM_P2_RGB_GAIN_GRN             FI_FPGA_IPM0_REG(0x01D0)
#define IPM_P2_RGB_GAIN_BLU             FI_FPGA_IPM0_REG(0x01D4)
#define IPM_P2_GT_DMA_START_ADDR        FI_FPGA_IPM0_REG(0x01D8)
#define IPM_P2_GT_DMA_BLOCK_SIZE        FI_FPGA_IPM0_REG(0x01DC)
// 0x1D8-0x1F7 are reserved.
#define IPM_P2_RAW_PIXEL_OFFSET_GAIN	FI_FPGA_IPM0_REG(0x01F8)
// 0x1FC-0x1FF are reserved.
#define IPM_IC_DMA_BURST_SIZE         	FI_FPGA_IPM0_REG(0x0200)
#define IPM_IC_DMA_BP_THRESHOLD       	FI_FPGA_IPM0_REG(0x0204)
// 0x208-0x27F are reserved.
#define IPM_P0_CCM_RED_DC_OFFSET	FI_FPGA_IPM0_REG(0x0280)
#define IPM_P0_CCM_GRN_DC_OFFSET	FI_FPGA_IPM0_REG(0x0284)
#define IPM_P0_CCM_BLU_DC_OFFSET	FI_FPGA_IPM0_REG(0x0288)
#define IPM_P1_CCM_RED_DC_OFFSET	FI_FPGA_IPM0_REG(0x028C)
#define IPM_P1_CCM_GRN_DC_OFFSET	FI_FPGA_IPM0_REG(0x0290)
#define IPM_P1_CCM_BLU_DC_OFFSET	FI_FPGA_IPM0_REG(0x0294)
#define IPM_P2_CCM_RED_DC_OFFSET	FI_FPGA_IPM0_REG(0x0298)
#define IPM_P2_CCM_GRN_DC_OFFSET	FI_FPGA_IPM0_REG(0x029C)
#define IPM_P2_CCM_BLU_DC_OFFSET	FI_FPGA_IPM0_REG(0x02A0)
// 0x2A4-0x2FF are reserved.
#define IPM_BP00_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0300)
#define IPM_BP00_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x0304)
#define IPM_BP01_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0308)
#define IPM_BP01_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x030C)
#define IPM_BP02_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0310)
#define IPM_BP02_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x0314)
#define IPM_BP03_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0318)
#define IPM_BP03_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x031C)
#define IPM_BP04_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0320)
#define IPM_BP04_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x0324)
#define IPM_BP05_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0328)
#define IPM_BP05_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x032C)
#define IPM_BP06_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0330)
#define IPM_BP06_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x0334)
#define IPM_BP07_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0338)
#define IPM_BP07_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x033C)
#define IPM_BP08_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0340)
#define IPM_BP08_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x0344)
#define IPM_BP09_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0348)
#define IPM_BP09_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x034C)

#define IPM_BP10_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0350)
#define IPM_BP10_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x0354)
#define IPM_BP11_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0358)
#define IPM_BP11_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x035C)
#define IPM_BP12_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0360)
#define IPM_BP12_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x0364)
#define IPM_BP13_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0368)
#define IPM_BP13_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x036C)
#define IPM_BP14_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0370)
#define IPM_BP14_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x0374)
#define IPM_BP15_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0378)
#define IPM_BP15_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x037C)
#define IPM_BP16_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0380)
#define IPM_BP16_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x0384)
#define IPM_BP17_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0388)
#define IPM_BP17_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x038C)
#define IPM_BP18_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0390)
#define IPM_BP18_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x0394)
#define IPM_BP19_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x0398)
#define IPM_BP19_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x039C)

#define IPM_BP20_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03A0)
#define IPM_BP20_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03A4)
#define IPM_BP21_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03A8)
#define IPM_BP21_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03AC)
#define IPM_BP22_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03B0)
#define IPM_BP22_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03B4)
#define IPM_BP23_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03B8)
#define IPM_BP23_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03BC)
#define IPM_BP24_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03C0)
#define IPM_BP24_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03C4)
#define IPM_BP25_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03C8)
#define IPM_BP25_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03CC)
#define IPM_BP26_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03D0)
#define IPM_BP26_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03D4)
#define IPM_BP27_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03D8)
#define IPM_BP27_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03DC)
#define IPM_BP28_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03E0)
#define IPM_BP28_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03E4)
#define IPM_BP29_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03E8)
#define IPM_BP29_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03EC)

#define IPM_BP30_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03F0)
#define IPM_BP30_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03F4)
#define IPM_BP31_CONFIG_A_ADDR  	FI_FPGA_IPM0_REG(0x03F8)
#define IPM_BP31_CONFIG_B_ADDR  	FI_FPGA_IPM0_REG(0x03FC)

// 0x2A4-0xFFF are reserved.
#define IPM_RGB_HISTO_TABLE_RED_START   FI_FPGA_IPM0_REG(0x1000)
#define IPM_RGB_HISTO_TABLE_GRN_START   FI_FPGA_IPM0_REG(0x1400)
#define IPM_RGB_HISTO_TABLE_BLU_START   FI_FPGA_IPM0_REG(0x1800)
#define IPM_RGB_HISTO_TABLE_SIZE	0x400

#define IPM_P0_BCG_LUT_START            FI_FPGA_IPM1_REG(0x0000)
#define IPM_P0_BCG_LUT_END              FI_FPGA_IPM1_REG(0x0FFC)
#define IPM_P1_BCG_LUT_START            FI_FPGA_IPM1_REG(0x1000)
#define IPM_P1_BCG_LUT_END              FI_FPGA_IPM1_REG(0x1FFC)
#define IPM_P2_BCG_LUT_START            FI_FPGA_IPM1_REG(0x2000)
#define IPM_P2_BCG_LUT_END              FI_FPGA_IPM1_REG(0x2FFC)
#define IPM_Px_BCG_LUT_SIZE		0x1000


//-----------------------------------------------
// IPM_DEST_DMA_BURST_SIZE register bit definitions
//-----------------------------------------------
#define IPM_DEST_DMA_BURST_TIMER_ENA	    ( 1 << 8 )
#define IPM_DEST_DMA_BURST_SIZE_SHIFT	    0
#define IPM_DEST_DMA_BURST_SIZE_MASK	    ( 0x3F << 0 )
#define IPM_DEST_DMA_BURST_PERIOD_SHIFT	    16
#define IPM_DEST_DMA_BURST_PERIOD_MASK	    ( 0xFFFF << 16 )

//-------------------------------------
// IPM_CONTROL register bit definitions
//-------------------------------------
#define ICTL_HISTO_START        (1 << 0)
#define ICTL_HISTO_PORT_MASK	(3 << 1)
#define ICTL_HISTO_PORT_SHIFT	1
#define ICTL_RGB_HISTO_RESET	(1 << 4)
#define ICTL_IPM_SW_RESET	(1 << 8)
#define ICTL_IC_RESET		(1 << 12)
#define ICTL_BCG_LUT_SEL_MASK	(3 << 16)
#define ICTL_BCG_LUT_SEL_SHIFT	16
#define ICTL_BCG_LUT_SEL_RED	(0 << ICTL_BCG_LUT_SEL_SHIFT)
#define ICTL_BCG_LUT_SEL_GREEN	(1 << ICTL_BCG_LUT_SEL_SHIFT)
#define ICTL_BCG_LUT_SEL_BLUE	(2 << ICTL_BCG_LUT_SEL_SHIFT)

//-------------------------------------
// IPM_STATUS register bit definitions
//-------------------------------------
#define IPMS_HISTO_BUSY	        (1 << 0)
#define IPMS_HISTO_INIT_BUSY    (1 << 1)
#define IPMS_IC_BUSY		(1 << 4)
#define IPMS_IPM_BUSY		(1 << 31)

//-------------------------------------
// IPM_Px_CONFIG register bit definitions
//-------------------------------------
#define IPMC_FPN_ENA		    (1 << 0)
#define IPMC_PRNU_ENA		    (1 << 1)
#define IPMC_PRNU_GAIN_SEL_0_8	    (0 << 2)
#define IPMC_PRNU_GAIN_SEL_1_7	    (1 << 2)
#define IPMC_SRC_FRM_COLOR_MODE	    (1 << 4)    // 1=color, only when dest_buf_type=1
#define IPMC_SRC_FRM_BAYER_PATTERN  (3 << 5)	// Bayer pattern if SRC_FRM_COLOR_MODE=1
#define 	IPMC_SRC_FRM_BAYER_GB	    (0 << 5)
#define 	IPMC_SRC_FRM_BAYER_BG	    (1 << 5)
#define 	IPMC_SRC_FRM_BAYER_GR	    (2 << 5)
#define 	IPMC_SRC_FRM_BAYER_RG	    (3 << 5)
#define IPMC_SRC_BUF_BIT_DEPTH_MASK (3 << 8)
#define		IPMC_SRC_BIT_DEPTH_8	    (0 << 8)
#define		IPMC_SRC_BIT_DEPTH_10	    (1 << 8)
#define		IPMC_SRC_BIT_DEPTH_12	    (2 << 8)
#define IPMC_SRC_BUF_BIT_SELECT_MASK	(7 << 12)   // Low bit, 0-4 for 8-bit, 0-2 for 10-bit
#define IPMC_SRC_BUF_BIT_SELECT_SHIFT	12	    // Shift for bit number
#define IPMC_DEST_TYPE_MASK	    (3 << 16)
#define 	IPMC_DEST_TYPE_MONO8	(0 << 16)
#define 	IPMC_DEST_TYPE_RGB24	(1 << 16)
#define 	IPMC_DEST_TYPE_RAW8	(2 << 16)
#define 	IPMC_DEST_TYPE_RAW16	(3 << 16)

//-------------------------------------
// IPM_Px_FRAME_DIM register definitions
//-------------------------------------
#define IPM_FRAME_DIM_Y_SHIFT 16

//-------------------------------------
// IPM_Px_BPy_CONFIG register bit definitions
//-------------------------------------
#define BP_CONFIG_B_ROW_SHIFT       0
#define BP_CONFIG_B_ROW_MASK        0x1FFF
#define BP_CONFIG_B_COL_SHIFT       16
#define BP_CONFIG_B_COL_MASK        0x1FFF0000

#define BP_CONFIG_A_OFFSET_SHIFT    0
#define BP_CONFIG_A_OFFSET_MASK     0x0F
#define BP_CONFIG_A_ENABLE          (1 << 31)

#define BP_CONFIG_A_VALUE( o, r, c)  ( BP_CONFIG_A_ENABLE | ( (o) << BP_CONFIG_A_OFFSET_SHIFT ) )
#define BP_CONFIG_B_VALUE( o, r, c)  ( ( (r) << BP_CONFIG_B_ROW_SHIFT ) | \
				       ( (c) << BP_CONFIG_B_COL_SHIFT ) )

//-------------------------------------
// Macros
//-------------------------------------

//! Macro to determine if this is a valid register address or not.
#define FI_FPGA_REG_VALID(x)	\
    (( FI_FPGA_BASE_ADDR <= (x)) && ( (x) < FI_FPGA_MAX_ADDR ))

#endif /* __FI_FPGA_REGS_H__ */

