/*! 
 * @file fi_fpga_dev.h
 * @brief Internal definitions for the Fastec Imaging FPGA driver.
 *
 * Copyright (C) 2017 Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef __FI_FPGA_DEV_H__
#define __FI_FPGA_DEV_H__

//-----------------------------------------------------------------------------
// Include Files
//-----------------------------------------------------------------------------

#include <linux/interrupt.h>	// Needed for the interrupt handler stuff.
#include <linux/cdev.h>		// Needed for our cdev base definitions.
#include <asm/atomic.h>		// Needed for atomic variables.

#include "fi_ioctl.h"

//-----------------------------------------------------------------------------
// Constant Definitions
//-----------------------------------------------------------------------------

//! The name we call ourself. 
#define MODULE_NAME 		"fi_fpga"

// Debug macros
#undef PDEBUG  // undef it, just in case.
#define PDEBUG(level, fmt, args...) \
    do { \
	if ( level <= fi_fpga_verbose ) \
	    { printk( KERN_DEBUG "fi_fpga: " fmt, ## args ); } \
    } while (0)

//! The Fastec Imaging FPGA private driver data structure.
typedef struct FI_FPGA_DEV {
    spinlock_t	reg_lock;	//!< Access control to the device.

    // Userland interface.
    struct cdev  	fpga_cdev;	//!< Char device structure.
    struct class 	*fpga_class;	//!< Our class structure.
    struct device 	*fpga_dev;	//!< Our device.
    struct device	*hdev;		//!< The hwmon device.
    dev_t		devno;		//!< Our device number.
    atomic_t		available;	//!< Number of available "opens".
    void 		*temp_buffer;	//!< Memory buffer (4096 bytes) for use as a "bounce buffer".

} FI_FPGA_DEV;

//-----------------------------------------------------------------------------
// Variable Declarations
//-----------------------------------------------------------------------------

extern int fi_fpga_verbose;

//-----------------------------------------------------------------------------
// Function Prototypes
//-----------------------------------------------------------------------------

// fpga_fops.c
extern int	fi_fpga_open( struct inode *inode, struct file *filp );
extern int	fi_fpga_release( struct inode *inode, struct file *filp );
extern long	fi_fpga_ioctl(struct file *filp, unsigned int cmd, unsigned long arg);
extern int	fi_fpga_image_read( FI_IMAGE_ACCESS *im_access );
extern int	fi_fpga_image_write( FI_IMAGE_ACCESS *im_access );

// fpga_hwmon.c
extern int	fi_fpga_setup_hdev(void);

// fpga_io.c
extern void	fi_set_fpga_reg( FI_FPGA_DEV *dev, uint32_t reg_id, uint32_t reg_value );
extern uint32_t fi_get_fpga_reg( FI_FPGA_DEV *dev, uint32_t reg_id );
extern uint32_t fi_mod_fpga_reg( FI_FPGA_DEV *dev, uint32_t reg_id, uint32_t reg_value, uint32_t reg_mask );

#endif /* __FI_FPGA_DEV_H__ */
