/*! 
 * @file fi_pci_dev.h
 * @brief Header file for the Fastec PCI device driver.
 *
 * Copyright (C) 2017 Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef _FASTEC_PCI_DEV_H
#define _FASTEC_PCI_DEV_H

// -----------------------------------------------------------------------
// Includes
// -----------------------------------------------------------------------
#include <linux/device.h>
#include <linux/cdev.h>
#include <linux/interrupt.h>
#include <linux/pci.h>

// Uncomment to use P2PMEM_ calls.
// #define USE_P2P 1

#ifdef USE_P2P
#include <linux/p2pmem.h>
#endif

// -----------------------------------------------------------------------
// Constant Definitions
// -----------------------------------------------------------------------
#define FASTEC_PCI_DRIVER_NAME    	"Fastec PCI"
#define FASTEC_PCI_BAR_NUM		6

#define MODULE_NAME 			"fi_pci"

typedef struct FASTEC_PCI_PRIVATE
{
    struct pci_dev *pci_dev;		// Pointer to our PCI device.

    u8 revision;
    u8 irq_pin;
    char msi_enabled;
    u8 irq_line;
    char dma_capable;

    u64 bar_start[FASTEC_PCI_BAR_NUM];
    void * __iomem io_bar[FASTEC_PCI_BAR_NUM];
    size_t bar_length[FASTEC_PCI_BAR_NUM];

    // Userland interface.
    struct cdev  	pci_cdev;	//!< Char device structure.
    struct class 	*pci_class;	//!< Our class structure.
    struct device 	*pci_cdev_dev;	//!< Our device for the char device class.
    dev_t		devno;		//!< Our device number.

#ifdef USE_P2P
    // P2PMEM interface.
    struct p2pmem_dev 	*p2pmem;	//!< Pointer to the p2pmem device.
#else
    u8 *virt_bar_addr;			//!< Virtual address of our selected bar.
    struct percpu_ref	ref;
    struct completion	cmp;
#endif

} FASTEC_PCI_PRIVATE;

// -----------------------------------------------------------------------
// Variable Declarations
// -----------------------------------------------------------------------
// Our private device structure.
extern FASTEC_PCI_PRIVATE *fi_pci_priv_ptr;

// -----------------------------------------------------------------------
// Function Prototypes
// -----------------------------------------------------------------------
extern int __init fastec_pci_probe(struct pci_dev *dev, const struct pci_device_id *id);
extern void __exit fastec_pci_remove(struct pci_dev *dev);

extern void __init fi_pci_proc_init(void * __iomem pci_bar0);
extern void __exit fi_pci_proc_exit(void);

extern int __init fi_pci_cdev_init(FASTEC_PCI_PRIVATE *priv_ptr);
extern void __exit fi_pci_cdev_exit(FASTEC_PCI_PRIVATE *priv_ptr);

extern irqreturn_t fi_pci_isr(int irq, void *dev_id);

#endif /* _FASTEC_PCI_DEV_H */
